from unittest.mock import MagicMock, Mock

from demo.models import Artist
from demo.urls import public_site
from django.contrib.contenttypes.models import ContentType
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.sessions.middleware import SessionMiddleware

from adminfilters.depot.models import StoredFilter
from adminfilters.depot.widget import DepotManager


def test_save(admin_user, rf):
    request = rf.get("/")
    request.user = admin_user
    for m in [SessionMiddleware, MessageMiddleware]:
        m(MagicMock()).process_request(request)

    f = DepotManager(
        request,
        {DepotManager.parameter_name: "Filter1"},
        None,
        public_site._registry[Artist],
    )
    f.queryset(request, None)
    assert StoredFilter.objects.filter(name="Filter1").exists()


def test_choices(admin_user, rf):
    request = rf.get("/?a=1")
    request.user = admin_user
    for m in [SessionMiddleware, MessageMiddleware]:
        m(MagicMock()).process_request(request)
    StoredFilter.objects.create(
        name="Filter1",
        content_type=ContentType.objects.get_for_model(Artist),
        owner=admin_user,
        query_string="?a=1",
    )
    StoredFilter.objects.create(
        name="Filter2",
        content_type=ContentType.objects.get_for_model(Artist),
        owner=admin_user,
        query_string="?a=2",
    )

    f = DepotManager(request, {"a": "1"}, None, public_site._registry[Artist])
    choices = list(f.choices(Mock()))
    assert len(choices) == 2
    assert choices[0] == {"name": "Filter1", "query_string": "?a=1", "selected": True}
    assert choices[1] == {"name": "Filter2", "query_string": "?a=2", "selected": False}



import pytest
from demo.models import DemoModelField

from adminfilters.dates import DateRangeFilter


@pytest.fixture
def fixtures(db):
    from demo.utils import DATA

    for i in range(1, 6):
        values = DATA.copy()
        values["unique"] = i
        values["datetime"] = "2000-01-%d 10:30:00-00:00" % i
        values["date"] = "2000-01-%d" % i
        values["json"] = {}

        DemoModelField.objects.create(**values)


@pytest.mark.parametrize("field", ("date", "datetime"))
@pytest.mark.parametrize(
    "op,expected",
    [
        ("2000-01-01", "2000-01-01"),
        (">2000-01-03", "2000-01-04,2000-01-05"),
        (">=2000-01-03", "2000-01-03,2000-01-04,2000-01-05"),
        ("<2000-01-03", "2000-01-01,2000-01-02"),
        ("<=2000-01-03", "2000-01-01,2000-01-02,2000-01-03"),
        ("2000-01-02..2000-01-04", "2000-01-02,2000-01-03,2000-01-04"),
        ("2000-01-03,2000-01-05", "2000-01-03,2000-01-05"),
        ("<>2000-01-03", "2000-01-01,2000-01-02,2000-01-04,2000-01-05"),
    ],
)
def test_NumberFilter(fixtures, field, op, expected):
    f = DateRangeFilter(
        DemoModelField._meta.get_field(field), None, {field: op}, None, None, field
    )
    assert f.value() == [op]
    result = f.queryset(None, DemoModelField.objects.all())
    value = [x.strftime("%Y-%m-%d") for x in result.values_list(field, flat=True)]
    assert value == expected.split(","), f.error_message


def test_factory(fixtures):
    assert DateRangeFilter.factory(title="CustomTitle")


def test_factory_invalid(fixtures):
    with pytest.raises(ValueError):
        assert DateRangeFilter.factory(title="CustomTitle", lookup_name="in")



import pytest
from demo.utils import ChangeListWrapper
from selenium.webdriver import Keys
from selenium.webdriver.common.by import By

pytestmark = pytest.mark.selenium


def get_elements(selenium):
    container = selenium.find_element(By.ID, "dj__key_dj__value_dj__negate")
    return [
        container,
        container.find_element(By.CSS_SELECTOR, "input[name=key]"),
        container.find_element(By.CSS_SELECTOR, "input[name=value]"),
        container.find_element(By.CSS_SELECTOR, "input[type=checkbox]"),
        container.find_element(By.CSS_SELECTOR, "a.button"),
        ChangeListWrapper.find_in_page(selenium),
    ]


@pytest.mark.selenium
def test_djangolookup_simple(admin_site):
    __, key, value, __, button, cl = get_elements(admin_site.driver)

    key.send_keys("name")
    value.send_keys("Angus")
    button.click()
    __, key, value, __, button, cl = get_elements(admin_site.driver)
    assert set(cl.get_values(None, 2)) == {"Angus"}


@pytest.mark.selenium
def test_djangolookup_negate(admin_site):
    __, key, value, negate, __, cl = get_elements(admin_site.driver)

    key.send_keys("name")
    value.send_keys("Angus")
    negate.click()
    value.send_keys(Keys.ENTER)
    __, key, value, negate, button, cl = get_elements(admin_site.driver)
    assert "Angus" not in set(cl.get_values(None, 2))


@pytest.mark.selenium
def test_djangolookup_invalid_field(admin_site):
    __, key, value, __, __, __ = get_elements(admin_site.driver)

    key.send_keys("xxx")
    value.send_keys("Angus")
    value.send_keys(Keys.ENTER)
    container, __, __, __, __, __ = get_elements(admin_site.driver)
    assert "Unknown field 'xxx'" in container.get_attribute("innerHTML")



from django.conf import settings
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.forms import Media
from django.urls import reverse
from django.utils.translation import get_language

from .mixin import MediaDefinitionFilter, SmartFieldListFilter


def get_real_field(model, path):
    parts = path.split("__")
    current = model
    for p in parts:
        f = current._meta.get_field(p)
        if f.related_model:
            current = f.related_model
    return f


class AutoCompleteFilter(SmartFieldListFilter, MediaDefinitionFilter):
    template = "adminfilters/autocomplete.html"
    filter_title = None
    parent = None
    parent_lookup_kwarg = None

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.dependants = []
        self.lookup_kwarg = "%s__exact" % field_path
        self.lookup_kwarg_isnull = "%s__isnull" % field_path
        self._params = params
        self.request = request
        super().__init__(field, request, params, model, model_admin, field_path)
        self.lookup_val = self.get_parameters(self.lookup_kwarg)
        self.admin_site = model_admin.admin_site
        self.query_string = ""
        self.target_field = get_real_field(model, field_path)
        self.target_model = self.target_field.related_model

        self.target_opts = self.target_field.model._meta

        # if not hasattr(field, "get_limit_choices_to"):
        #     raise Exception(
        #         f"Filter '{field_path}' of {model_admin} is not supported by AutoCompleteFilter."
        #         f" Check your {model_admin}.list_filter value"
        #     )

        self.url = self.get_url()

    def expected_parameters(self):
        return [self.lookup_kwarg, self.lookup_kwarg_isnull]

    def get_url(self):
        return reverse("%s:autocomplete" % self.admin_site.name)

    def choices(self, changelist):
        self.query_string = changelist.get_query_string(
            remove=[self.lookup_kwarg, self.lookup_kwarg_isnull]
        )
        if self.lookup_val:
            get_kwargs = {self.field.target_field.name: self.lookup_val}
            return [str(self.target_model.objects.get(**get_kwargs)) or ""]
        return []

    @property
    def media(self):
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (
            ("admin/js/vendor/select2/i18n/%s.js" % i18n_name,) if i18n_name else ()
        )
        return Media(
            js=(
                "admin/js/vendor/jquery/jquery%s.js" % extra,
                "admin/js/vendor/select2/select2.full%s.js" % extra,
            )
            + i18n_file
            + (
                "admin/js/jquery.init.js",
                "admin/js/autocomplete.js",
                "adminfilters/autocomplete%s.js" % extra,
            ),
            css={
                "screen": (
                    "admin/css/vendor/select2/select2%s.css" % extra,
                    "adminfilters/adminfilters.css",
                ),
            },
        )

    @classmethod
    def factory(cls, *, title=None, lookup_name="exact", **kwargs):
        kwargs["filter_title"] = title
        kwargs["lookup_name"] = lookup_name
        return type("ValueFilter", (cls,), kwargs)

    def get_title(self):
        if not self.can_negate and self.negated:
            if self.negated_title:
                return self.negated_title
            else:
                return f"not {self.title}"
        return self.filter_title or self.title


class LinkedAutoCompleteFilter(AutoCompleteFilter):
    parent = None
    parent_lookup_kwarg = None
    extras = []
    dependants = []

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.dependants = []
        if self.parent:
            self.parent_lookup_kwarg = f"{self.parent}__exact"
        super().__init__(field, request, params, model, model_admin, field_path)
        for pos, entry in enumerate(model_admin.list_filter):
            if isinstance(entry, (list, tuple)):
                if (
                    len(entry) == 2
                    and entry[0] != self.field_path
                    and entry[1].__name__ == type(self).__name__
                    and entry[1].parent == self.field_path
                ):
                    kwarg = f"{entry[0]}__exact"
                    if entry[1].parent:
                        if kwarg not in self.dependants:
                            self.dependants.extend(entry[1].dependants)
                            self.dependants.append(kwarg)

    def has_output(self):
        if self.parent:
            return self.parent_lookup_kwarg in self.request.GET
        return True

    def choices(self, changelist):
        to_remove = [self.lookup_kwarg, self.lookup_kwarg_isnull]
        p = changelist.params.copy()
        for f in p:
            if f in self.dependants:
                to_remove.append(f)

        self.query_string = changelist.get_query_string(remove=to_remove)
        if self.lookup_val:
            return [str(self.target_model.objects.get(pk=self.lookup_val)) or ""]
        return []

    def get_url(self):
        url = reverse("%s:autocomplete" % self.admin_site.name)
        if self.parent_lookup_kwarg in self.request.GET:
            flt = self.parent_lookup_kwarg.split("__")[-2]
            oid = self.request.GET[self.parent_lookup_kwarg]
            return f"{url}?{flt}={oid}"
        return url

    @classmethod
    def factory(cls, *, title=None, lookup_name="exact", **kwargs):
        kwargs["filter_title"] = title
        kwargs["lookup_name"] = lookup_name
        return type("LinkedAutoCompleteFilter", (cls,), kwargs)



from django.contrib.auth.models import Permission

from adminfilters.compat import DJANGO_MAJOR
from adminfilters.extra import PermissionPrefixFilter


def test_GenericLookupFieldFilter(db):
    f = PermissionPrefixFilter

    if DJANGO_MAJOR < 5:
        param = "add"
    else:
        param = ["add"]
    qs = f(None, {"perm": param}, None, None).queryset(None, Permission.objects.all())
    assert qs.first().codename.startswith("add_")

    if DJANGO_MAJOR < 5:
        param = "change"
    else:
        param = ["change"]
    qs = f(None, {"perm": param}, None, None).queryset(
        None, Permission.objects.all()
    )
    assert qs.first().codename.startswith("change_")

    if DJANGO_MAJOR < 5:
        param = "delete"
    else:
        param = ["delete"]
    qs = f(None, {"perm": param}, None, None).queryset(
        None, Permission.objects.all()
    )
    assert qs.first().codename.startswith("delete_")

    qs = f(None, {}, None, None).queryset(None, Permission.objects.all())
    assert qs.exists()

    if DJANGO_MAJOR < 5:
        param = "--"
    else:
        param = ["--"]
    qs = f(None, {"perm": param}, None, None).queryset(None, Permission.objects.all())
    assert not qs.exists()



from unittest.mock import Mock

import pytest
from demo.admin import ArtistModelAdmin
from demo.factories import ArtistFactory
from demo.management.commands.init_demo import sample_data
from demo.models import Artist
from django.contrib.admin import site

from adminfilters.checkbox import RelatedFieldCheckBoxFilter


@pytest.fixture
def fixtures(db):
    ArtistFactory(name="name1", bands=[])
    return sample_data()


def test_RelatedFieldCheckBoxFilte_single(fixtures, rf):
    acdc, geordie = fixtures
    request = rf.get(f"/?bands__id={geordie.id}")
    f = RelatedFieldCheckBoxFilter(
        Artist._meta.get_field("bands"),
        request,
        {},
        None,
        ArtistModelAdmin(Artist, site),
        "bands",
    )
    result = f.queryset(None, Artist.objects.all())

    value = sorted(list(result.values_list("name", flat=True)))
    assert value == ["Brian"]


def test_RelatedFieldCheckBoxFilte_multi(fixtures, rf):
    acdc, geordie = fixtures
    request = rf.get(f"/?bands__id={acdc.id}&bands_id={geordie.id}")
    f = RelatedFieldCheckBoxFilter(
        Artist._meta.get_field("bands"),
        request,
        {},
        None,
        ArtistModelAdmin(Artist, site),
        "bands",
    )
    result = f.queryset(None, Artist.objects.all())

    value = sorted(list(result.values_list("name", flat=True)))
    assert value == ["Angus", "Bon", "Brian", "Malcom", "Phil"]


def test_RelatedFieldCheckBoxFilte_isnull(fixtures, rf):
    request = rf.get("/?bands__isnull=true")
    f = RelatedFieldCheckBoxFilter(
        Artist._meta.get_field("bands"),
        request,
        {"bands__isnull": "true"},
        None,
        ArtistModelAdmin(Artist, site),
        "bands",
    )
    result = f.queryset(None, Artist.objects.all())

    value = sorted(list(result.values_list("name", flat=True)))
    assert value == ["name1"]


def test_RelatedFieldCheckBoxFilte_choices(fixtures, rf):
    request = rf.get("/")
    f = RelatedFieldCheckBoxFilter(
        Artist._meta.get_field("bands"),
        request,
        {},
        None,
        ArtistModelAdmin(Artist, site),
        "bands",
    )
    choices = list(f.choices(Mock()))
    assert len(choices) == 4
    assert [c["display"] for c in choices] == ["All", "None", "AC/DC", "Geordie"]



from django import forms
from django.contrib.admin import FieldListFilter, ListFilter
from django.contrib.admin.options import ModelAdmin
from django.core import checks

from adminfilters.compat import DJANGO_MAJOR


class WrappperMixin:
    negated = False
    can_negate = False
    title = None
    negated_title = None
    placeholder = ""

    def __init__(self, *args, **kwargs) -> None:
        self.error = None
        self.error_message = None
        super().__init__(*args, **kwargs)
        if (
            hasattr(self, "media")
            and self.model_admin
            and not isinstance(self.model_admin, AdminFiltersMixin)
        ):
            raise Exception(
                f"{self.model_admin.__class__.__name__} must inherit from AdminFiltersMixin"
            )

    def get_parameters(self, param_name, default="", multi=False, pop=False, separator=","):
        if pop:
            val = self._params.pop(param_name, default)
        else:
            val = self._params.get(param_name, default)
        if val:
            if DJANGO_MAJOR >= 5:
                if isinstance(val, list) and not multi:
                    val = val[-1]
            elif multi:
                val = val.split(separator)
        return val

    def html_attrs(self):
        classes = f"adminfilters box {self.__class__.__name__.lower()}"
        if self.error_message:
            classes += " error"

        return {
            "class": classes,
            "id": "_".join(self.expected_parameters()),
        }

    def get_title(self):
        if not self.can_negate and self.negated:
            if self.negated_title:
                return self.negated_title
            else:
                return f"not {self.title}"
        return self.title


class SmartListFilter(WrappperMixin, ListFilter):
    def __init__(self, request, params, model, model_admin):
        self.model_admin = model_admin
        self._params = params
        super().__init__(request, params, model, model_admin)


class SmartFieldListFilter(WrappperMixin, FieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        self.model_admin = model_admin
        self._params = params
        super().__init__(field, request, params, model, model_admin, field_path)


class MediaDefinitionFilter:
    pass


class AdminFiltersMixin(ModelAdmin):
    def _check_linked_fields_modeladmin(self):
        from .autocomplete import LinkedAutoCompleteFilter

        linked_filters = [
            e
            for e in self.list_filter
            if isinstance(e, (list, tuple))
            and issubclass(e[1], LinkedAutoCompleteFilter)
        ]
        errs = []
        seen = []
        for pos, entry in enumerate(linked_filters):
            if entry[1] and entry[1].parent:
                parts = entry[1].parent.split("__")
                m = self.model
                for part in parts:
                    m = m._meta.get_field(part).remote_field.model
                    ma: ModelAdmin = self.admin_site._registry[m]
                    if ma not in seen and not isinstance(
                        ma, AdminAutoCompleteSearchMixin
                    ):
                        errs.append(
                            checks.Error(
                                f"{ma}` must inherits from AdminAutoCompleteSearchMixin",
                                obj=ma.__class__,
                                id="admin.E041",
                            )
                        )
                        seen.append(ma)

        return errs

    def _check_linked_fields_order(self):
        from .autocomplete import LinkedAutoCompleteFilter

        linked_filters = [
            e
            for e in self.list_filter
            if isinstance(e, (list, tuple))
            and issubclass(e[1], LinkedAutoCompleteFilter)
        ]
        errs = []
        seen = []
        for pos, entry in enumerate(linked_filters):
            if entry[1] and entry[1].parent and entry[1].parent not in seen:
                errs.append(
                    checks.Error(
                        f"Invalid Filters ordering. '{entry[1].parent}' must be defined before '{entry[0]}'.",
                        obj=self.__class__,
                        id="admin.E040",
                    )
                )
            seen.append(entry[0])
        return errs

    def check(self, **kwargs):
        return [
            *super().check(**kwargs),
            *self._check_linked_fields_order(),
            *self._check_linked_fields_modeladmin(),
        ]

    def get_changelist_instance(self, request):
        cl = super().get_changelist_instance(request)
        for flt in cl.filter_specs:
            if hasattr(flt, "media"):
                self.admin_filters_media += flt.media
        return cl

    def __init__(self, model, admin_site):
        self.admin_filters_media = forms.Media()
        super().__init__(model, admin_site)

    @property
    def media(self):
        original = super().media
        if hasattr(self, "admin_filters_media"):
            original += self.admin_filters_media
        return original


class AdminAutoCompleteSearchMixin(ModelAdmin):
    def get_search_results(self, request, queryset, search_term):
        field_names = [f.name for f in self.model._meta.get_fields()]
        filters = {k: v for k, v in request.GET.items() if k in field_names}
        queryset = queryset.filter(**filters)
        queryset, may_have_duplicates = super().get_search_results(
            request, queryset, search_term
        )
        return queryset, may_have_duplicates



from django import forms
from django.conf import settings
from django.contrib.admin import ListFilter
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import get_language

from adminfilters.depot.models import StoredFilter
from adminfilters.utils import get_query_string

from ..mixin import WrappperMixin


class DepotManager(WrappperMixin, ListFilter):
    title = "Saved Filters"
    template = "adminfilters/widget.html"
    parameter_name = "adminfilters_depot"
    parameter_name_op = "adminfilters_depot_op"

    def __init__(self, request, params, model, model_admin):
        self.model_admin = model_admin
        self._params = params.copy()
        super().__init__(request, params, model, model_admin)
        self.request = request
        self.query_string = get_query_string(
            self.request, {}, self.expected_parameters()
        )
        self.can_add_filter = request.user.has_perm("depot.add_storedfilter")
        self.content_type = ContentType.objects.get_for_model(model_admin.model)
        for p in self.expected_parameters():
            if p in params:
                self.used_parameters[p] = params.pop(p)

    def has_output(self):
        return True

    def expected_parameters(self):
        return [self.parameter_name, self.parameter_name_op]

    def queryset(self, request, queryset):
        filter_name = self.get_parameters(self.parameter_name)
        operation = self.get_parameters(self.parameter_name_op, "add")
        if filter_name:
            if operation == "add":
                qs = get_query_string(request, {}, self.expected_parameters())

                StoredFilter.objects.update_or_create(
                    content_type=self.content_type,
                    name=filter_name,
                    defaults={
                        "query_string": qs,
                        "owner": request.user,
                    },
                )
                self.model_admin.message_user(
                    self.request, f"Filter '{filter_name}' successfully saved"
                )
            elif operation == "delete":
                StoredFilter.objects.filter(id=filter_name).delete()
        return queryset

    def choices(self, changelist):
        self.selected = False
        for f in StoredFilter.objects.filter(content_type=self.content_type).order_by(
            "name"
        ):
            if str(self.query_string) == str(f.query_string):
                self.selected = True
                self.selected_id = f.pk

            yield {
                "selected": str(self.query_string) == str(f.query_string),
                "query_string": f.query_string,
                "name": f.name,
            }

    @property
    def media(self):
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (
            ("admin/js/vendor/select2/i18n/%s.js" % i18n_name,) if i18n_name else ()
        )
        return forms.Media(
            js=("admin/js/vendor/jquery/jquery%s.js" % extra,)
            + i18n_file
            + (
                "admin/js/jquery.init.js",
                "adminfilters/depot%s.js" % extra,
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )



import django.contrib.admin.sites
import django.views.static
from django.conf import settings
from django.contrib.auth.models import User
from django.urls import re_path

from adminfilters.depot.admin import StoredFilterAdmin
from adminfilters.depot.models import StoredFilter

from . import admin, models


class PublicAdminSite(django.contrib.admin.sites.AdminSite):
    def has_permission(self, request):
        request.user = User.objects.get_or_create(username="sax")[0]
        return True


public_site = PublicAdminSite()
django.contrib.admin.autodiscover()
public_site.register(
    models.Artist_RelatedFieldCheckBoxFilter,
    admin.DemoModelAdmin_RelatedFieldCheckBoxFilter,
)
public_site.register(
    models.Artist_RelatedFieldRadioFilter, admin.DemoModelAdmin_RelatedFieldRadioFilter
)
public_site.register(
    models.Artist_UnionFieldListFilter, admin.DemoModelAdmin_UnionFieldListFilter
)
public_site.register(
    models.Artist_IntersectionFieldListFilter,
    admin.DemoModelAdmin_IntersectionFieldListFilter,
)
public_site.register(models.Country, admin.CountryModelAdmin)
public_site.register(models.Band, admin.BandModelAdmin)
public_site.register(models.Artist, admin.ArtistModelAdmin)
public_site.register(models.DemoModelField, admin.DemoModelFieldAdmin)
public_site.register(User, admin.IUserAdmin)
public_site.register(StoredFilter, StoredFilterAdmin)

urlpatterns = (
    re_path(
        r"^(?P<path>favicon.ico)$",
        django.views.static.serve,
        {"document_root": settings.STATIC_ROOT},
    ),
    re_path(
        r"^static/(?P<path>.*)$",
        django.views.static.serve,
        {"document_root": settings.STATIC_ROOT},
    ),
    re_path(r"", public_site.urls),
)



import os
import sys
from pathlib import Path
from uuid import uuid4

from environ import environ

here = os.path.dirname(__file__)
sys.path.append(os.path.abspath(os.path.join(here, "..", "..")))

BASE_DIR = Path(__file__).resolve(strict=True).parents[3]

env = environ.Env(
    DEBUG=(bool, False),
    STATIC_ROOT=(str, str(BASE_DIR / "~build" / "staticfiles")),
    DATABASE_URL=(str, ""),
    ROOT_TOKEN=(str, uuid4().hex),
)

DEBUG = env("DEBUG")
TEMPLATE_DEBUG = DEBUG
ALLOWED_HOSTS = ["*"]

TIME_ZONE = "Asia/Bangkok"
LANGUAGE_CODE = "en-us"
SITE_ID = 1
USE_I18N = True
USE_L10N = True
USE_TZ = True
MEDIA_ROOT = os.path.join(here, "media")
MEDIA_URL = "/media/"
STATIC_ROOT = env("STATIC_ROOT")
STATIC_URL = "/static/"
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

STATICFILES_DIRS = ()

SECRET_KEY = "c73*n!y=)tziu^2)y*@5i2^)$8z$tx#b9*_r3i6o1ohxo%*2^a"
MIDDLEWARE = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)

DATABASES = {"default": env.db()}
MESSAGE_STORAGE = "django.contrib.messages.storage.cookie.CookieStorage"

ROOT_URLCONF = "demo.urls"
WSGI_APPLICATION = "demo.wsgi.application"

AUTHENTICATION_BACKENDS = ("demo.backends.AnonymousAccessUserBackend",)

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "adminfilters",
    "adminfilters.depot",
    "demo",
)

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "debug": DEBUG,
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.request",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    }
]

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
        },
    },
    "root": {
        "handlers": ["console"],
        "level": "ERROR",
    },
    "pytest_selenium": {
        "handlers": ["console"],
        "level": "ERROR",
    },
}



import pytest
from demo.models import Artist

from adminfilters.compat import DJANGO_MAJOR
from adminfilters.filters import MultiValueFilter, ValueFilter


@pytest.fixture
def fixtures(db):
    from demo.factories import ArtistFactory

    ArtistFactory(name="a1")
    ArtistFactory(name="a2")
    ArtistFactory(name="b1")
    ArtistFactory(name="c1")


def test_media():
    assert ValueFilter.factory(title="Title")(
        None, None, {}, None, None, "unique"
    ).media


@pytest.mark.parametrize(
    "value,negate,expected",
    [
        ("n", False, []),
        ("a1", False, ["a1"]),
        ("a1", True, ["a2", "b1", "c1"]),
    ],
)
def test_value_filter(fixtures, value, negate, expected):
    f = ValueFilter(
        Artist._meta.get_field("name"),
        None,
        {"name__exact": value, "name__exact__negate": str(negate).lower()},
        None,
        None,
        "name",
    )

    result = f.queryset(None, Artist.objects.all())
    value = list(result.values_list("name", flat=True))
    assert value == expected


def test_factory(fixtures):
    F = ValueFilter.factory(title="CustomTitle")
    f = F(
        Artist._meta.get_field("name"),
        None,
        {"name__exact": "a1", "name__exact__negate": "false"},
        None,
        None,
        "name",
    )

    assert f.value() == ["a1", False]
    result = f.queryset(None, Artist.objects.all())
    value = list(result.values_list("name", flat=True))
    assert value == ["a1"]


@pytest.mark.parametrize(
    "value,negate,expected",
    [
        ("n", False, []),
        ("a1", False, ["a1"]),
        ("a1", True, ["a2", "b1", "c1"]),
    ],
)
def test_MultiValueTextFieldFilter(fixtures, value, negate, expected):
    if DJANGO_MAJOR < 5:
        val = value
    else:
        val = [value]
    f = MultiValueFilter(
        Artist._meta.get_field("name"),
        None,
        {"name__in": val, "name__in__negate": str(negate).lower()},
        None,
        None,
        "name",
    )
    assert f.value() == [[value], negate]
    result = f.queryset(None, Artist.objects.all())
    value = list(result.values_list("name", flat=True))
    assert value == expected



import os
import sys
from pathlib import Path


def pytest_addoption(parser):
    parser.addoption(
        "--selenium",
        action="store_true",
        dest="enable_selenium",
        default=False,
        help="enable selenium tests",
    )

    parser.addoption(
        "--show-browser",
        "-S",
        action="store_true",
        dest="show_browser",
        default=False,
        help="will display start browsers in selenium tests",
    )


def pytest_configure(config):
    sys._called_from_pytest = True
    sys.path.insert(0, str(Path(__file__).parent.parent / "src"))
    setattr(config.option, "driver", "chrome")

    if config.option.show_browser:
        setattr(config.option, "enable_selenium", True)

    if not config.option.enable_selenium:
        setattr(config.option, "markexpr", "not selenium")

    config.addinivalue_line(
        "markers", "skip_if_ci: this mark skips the tests on GitlabCI"
    )
    config.addinivalue_line(
        "markers", "skip_test_if_env(env): this mark skips the tests for the given env"
    )

    sys.path.insert(0, os.path.join(os.path.dirname(__file__), "demoapp"))
    os.environ["DJANGO_SETTINGS_MODULE"] = "demo.settings"
    import django

    django.setup()



import pytest
from demo.models import Artist

from adminfilters.dj import DjangoLookupFilter


@pytest.fixture
def fixtures(db):
    from demo.factories import ArtistFactory

    ArtistFactory(name="a1", active=True)
    ArtistFactory(name="a2", active=True)
    ArtistFactory(name="b1", active=True)
    ArtistFactory(name="c1", active=False)


def test_media():
    assert DjangoLookupFilter.factory(title="Title")(None, {}, Artist, None).media


@pytest.mark.parametrize(
    "key,value,negate,expected",
    [
        ("name", "a1", False, ["a1"]),
        ("name__endswith", "2", False, ["a2"]),
        ("name__in", "a1,a2", False, ["a1", "a2"]),
        ("active", "true", False, ["a1", "a2", "b1"]),
        ("active", "false", False, ["c1"]),
        ("active", "false", True, ["a1", "a2", "b1"]),
        ("xx", "zz", True, ["a1", "a2", "b1", "c1"]),
    ],
)
def test_value_filter(fixtures, key, value, negate, expected):
    f = DjangoLookupFilter(
        None,
        {"dj__key": key, "dj__value": value, "dj__negate": str(negate).lower()},
        Artist,
        None,
    )

    result = f.queryset(None, Artist.objects.all())
    value = list(result.values_list("name", flat=True))
    assert value == expected



import pytest
from demo.models import DemoModelField

from adminfilters.numbers import NumberFilter


@pytest.fixture
def fixtures(db):
    from demo.utils import DATA

    for i in range(1, 5):
        values = DATA.copy()
        values["unique"] = i
        values["json"] = {}
        DemoModelField.objects.create(**values)


@pytest.mark.parametrize(
    "op,expected",
    [
        ("=1", "1"),
        ("1", "1"),
        (">1", "2,3,4"),
        (">=1", "1,2,3,4"),
        ("<>2", "1,3,4"),
        ("<2", "1"),
        ("<=2", "1,2"),
        ("1,3", "1,3"),
        ("2..4", "2,3,4"),
    ],
)
def test_NumberFilter(fixtures, op, expected):
    f = NumberFilter(
        DemoModelField._meta.get_field("unique"),
        None,
        {"unique": op},
        None,
        None,
        "unique",
    )
    assert f.value() == [op]
    result = f.queryset(None, DemoModelField.objects.all())
    value = list(result.values_list("unique", flat=True))
    assert value == expected.split(","), f.error_message


def test_factory(fixtures):
    assert NumberFilter.factory(title="CustomTitle")


def test_factory_invalid(fixtures):
    with pytest.raises(ValueError):
        assert NumberFilter.factory(title="CustomTitle", lookup_name="in")



from .autocomplete import AutoCompleteFilter
from .checkbox import RelatedFieldCheckBoxFilter
from .combo import AllValuesComboFilter, ChoicesFieldComboFilter, RelatedFieldComboFilter
from .dj import DjangoLookupFilter
from .extra import PermissionPrefixFilter
from .json import JsonFieldFilter
from .multiselect import IntersectionFieldListFilter, UnionFieldListFilter
from .numbers import MaxMinFilter, NumberFilter
from .querystring import QueryStringFilter
from .radio import AllValuesRadioFilter, BooleanRadioFilter, ChoicesFieldRadioFilter, RelatedFieldRadioFilter
from .value import MultiValueFilter, ValueFilter



# Generated by Django 4.0.2 on 2023-05-26 16:30

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Band",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                (
                    "genre",
                    models.IntegerField(
                        choices=[(1, "Rock"), (2, "Blues"), (3, "Soul"), (4, "Other")]
                    ),
                ),
                ("active", models.BooleanField(default=True)),
            ],
        ),
        migrations.CreateModel(
            name="Country",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
            ],
            options={
                "verbose_name_plural": "Countries",
                "ordering": ("name",),
            },
        ),
        migrations.CreateModel(
            name="DemoModelField",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("flags", models.JSONField(blank=True, default=dict, null=True)),
                ("char", models.CharField(max_length=255)),
                ("integer", models.IntegerField()),
                ("logic", models.BooleanField(default=False)),
                ("date", models.DateField()),
                ("datetime", models.DateTimeField()),
                ("time", models.TimeField()),
                ("decimal", models.DecimalField(decimal_places=3, max_digits=10)),
                ("email", models.EmailField(max_length=254)),
                ("float", models.FloatField()),
                ("bigint", models.BigIntegerField()),
                ("generic_ip", models.GenericIPAddressField()),
                ("url", models.URLField()),
                ("text", models.TextField()),
                ("json", models.JSONField()),
                ("unique", models.CharField(max_length=255, unique=True)),
                ("nullable", models.CharField(max_length=255, null=True)),
                ("blank", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "not_editable",
                    models.CharField(
                        blank=True, editable=False, max_length=255, null=True
                    ),
                ),
                (
                    "choices",
                    models.IntegerField(
                        choices=[(1, "Choice 1"), (2, "Choice 2"), (3, "Choice 3")]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Artist",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("flags", models.JSONField(blank=True, default=dict, null=True)),
                ("name", models.CharField(max_length=255)),
                ("last_name", models.CharField(max_length=255)),
                ("full_name", models.CharField(max_length=255)),
                ("year_of_birth", models.IntegerField()),
                ("active", models.BooleanField(default=True)),
                (
                    "bands",
                    models.ManyToManyField(
                        related_name="bands", to="demo.Band", verbose_name="Bands"
                    ),
                ),
                (
                    "country",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="demo.country"
                    ),
                ),
            ],
            options={
                "ordering": ("name",),
            },
        ),
        migrations.CreateModel(
            name="Artist_IntersectionFieldListFilter",
            fields=[],
            options={
                "verbose_name": "IntersectionFieldListFilter",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("demo.artist",),
        ),
        migrations.CreateModel(
            name="Artist_RelatedFieldCheckBoxFilter",
            fields=[],
            options={
                "verbose_name": "RelatedFieldCheckBoxFilter",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("demo.artist",),
        ),
        migrations.CreateModel(
            name="Artist_RelatedFieldRadioFilter",
            fields=[],
            options={
                "verbose_name": "RelatedFieldRadioFilter",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("demo.artist",),
        ),
        migrations.CreateModel(
            name="Artist_UnionFieldListFilter",
            fields=[],
            options={
                "verbose_name": "UnionFieldListFilter",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("demo.artist",),
        ),
    ]



from django.contrib.admin.utils import prepare_lookup_value
from django.db.models.fields import AutoField, BigAutoField, IntegerField
from django.utils.translation import gettext_lazy as _

from adminfilters.mixin import SmartFieldListFilter


class MultipleSelectFieldListFilter(SmartFieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        self.lookup_kwarg = "%s_filter" % field_path
        self.filter_statement = "%s" % field_path
        self.lookup_val = params.pop(self.lookup_kwarg, None)
        self.lookup_choices = field.get_choices(include_blank=False)
        super().__init__(field, request, params, model, model_admin, field_path)
        self.used_parameters[self.lookup_kwarg] = prepare_lookup_value(
            self.lookup_kwarg, self.lookup_val
        )

    def expected_parameters(self):
        return [self.lookup_kwarg]

    def get_field(self):
        return self.field.remote_field.model._meta.pk

    def values(self):
        """
        Returns a list of values to filter on.
        """
        values = []
        value = self.used_parameters.get(self.lookup_kwarg, None)
        if value:
            values = value.split(",")

        field = self.get_field()
        # convert to integers if IntegerField
        if type(field) in [IntegerField, AutoField, BigAutoField]:
            values = [int(x) for x in values]
        return values

    def queryset(self, request, queryset):
        raise NotImplementedError

    def choices(self, cl):
        # from django.contrib.admin.views.main import EMPTY_CHANGELIST_VALUE
        yield {
            "selected": self.lookup_val is None,
            "query_string": cl.get_query_string({}, [self.lookup_kwarg]),
            "display": _("All"),
        }
        for pk_val, val in self.lookup_choices:
            selected = pk_val in self.values()
            pk_list = set(self.values())
            if selected:
                pk_list.remove(pk_val)
            else:
                pk_list.add(pk_val)
            queryset_value = ",".join([str(x) for x in pk_list])
            if pk_list:
                query_string = cl.get_query_string(
                    {
                        self.lookup_kwarg: queryset_value,
                    }
                )
            else:
                query_string = cl.get_query_string({}, [self.lookup_kwarg])
            yield {
                "selected": selected,
                "query_string": query_string,
                "display": val,
            }


class IntersectionFieldListFilter(MultipleSelectFieldListFilter):
    """
    A FieldListFilter which allows multiple selection of
    filters for many-to-many type fields. A list of objects will be
    returned whose m2m contains all the selected filters.
    """

    def queryset(self, request, queryset):
        for value in self.values():
            filter_dct = {self.filter_statement: value}
            queryset = queryset.filter(**filter_dct)
        return queryset


class UnionFieldListFilter(MultipleSelectFieldListFilter):
    """
    A FieldListFilter which allows multiple selection of
    filters for many-to-many type fields, or any type with choices.
    A list of objects will be returned whose m2m or value set
    contains one of the selected filters.
    """

    def get_field(self):
        try:
            field = super().get_field()
        except AttributeError:  # pragma: no cover
            if hasattr(self.field, "choices") and self.field.choices:
                field = self.field  # It's a *Field with choises
            else:
                raise AttributeError(
                    "Multiselect field must be a FK or any type with choices"
                )
        return field

    def queryset(self, request, queryset):
        filter_values = self.values()
        if filter_values:
            filter_statement = "%s__in" % self.filter_statement
            filter_dct = {filter_statement: filter_values}
            queryset = queryset.filter(**filter_dct).distinct()
        return queryset



import pytest
from demo.utils import ChangeListWrapper
from selenium.webdriver import Keys
from selenium.webdriver.common.by import By

pytestmark = pytest.mark.selenium


def get_elements(selenium):
    container = selenium.find_element(By.ID, "year_of_birth")
    return [
        container.find_element(By.CSS_SELECTOR, "input[type=text]"),
        container.find_element(By.CSS_SELECTOR, "a.button"),
        ChangeListWrapper.find_in_page(selenium),
    ]


@pytest.mark.parametrize(
    "value,expected",
    [
        ("1955", lambda n: n == 1955),
        (">1953", lambda n: n > 1953),
        ("<>1955", lambda n: n != 1955),
        ("<1947", lambda n: n != 1947),
        ("<=1947", lambda n: n <= 1947),
        ("1955,1953", lambda n: n in [1953, 1955]),
        ("1953..1955", lambda n: n in range(1953, 1955 + 1)),
    ],
    ids=["=", ">1", "<>", "<", "<=", "list", "range"],
)
@pytest.mark.selenium
def test_number_filter(admin_site, value, expected):
    input_text, button, cl = get_elements(admin_site.driver)
    input_text.send_keys(value, Keys.META, Keys.ENTER)
    input_text, button, cl = get_elements(admin_site.driver)
    years = list(map(int, cl.get_values(None, 6)))
    filtered = list(filter(expected, years))
    assert years == filtered



from django import forms
from django.conf import settings
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.db.models import Q
from django.utils.translation import get_language
from django.utils.translation import gettext as _

from .mixin import MediaDefinitionFilter, SmartFieldListFilter


class JsonFieldFilter(MediaDefinitionFilter, SmartFieldListFilter):
    parameter_name = None
    title = None
    template = "adminfilters/json.html"
    can_negate = True
    negated = False
    options = True
    key_placeholder = _("JSON key")
    placeholder = _("JSON value")

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.lookup_kwarg_key = "%s__key" % field_path
        self.lookup_kwarg_value = "%s__value" % field_path
        self.lookup_kwarg_negated = "%s__negate" % field_path
        self.lookup_kwarg_options = "%s__options" % field_path
        self.lookup_kwarg_type = "%s__type" % field_path
        self.lookup_key_val = params.get(self.lookup_kwarg_key, "")
        self.lookup_value_val = params.get(self.lookup_kwarg_value, "")
        self.lookup_negated_val = params.get(self.lookup_kwarg_negated, "false")
        self.lookup_options_val = params.get(self.lookup_kwarg_options, "e")
        self.lookup_type_val = params.get(self.lookup_kwarg_type, "any")

        self.field = field
        self.query_string = None
        self.field_path = field_path
        self.title = getattr(field, "verbose_name", field_path)
        super().__init__(field, request, params, model, model_admin, field_path)

    @classmethod
    def factory(cls, **kwargs):
        return type("JsonFieldFilter", (cls,), kwargs)

    def expected_parameters(self):
        return [
            self.lookup_kwarg_key,
            self.lookup_kwarg_value,
            self.lookup_kwarg_negated,
            self.lookup_kwarg_options,
            self.lookup_kwarg_type,
        ]

    def value(self):
        return [
            self.lookup_key_val,
            self.lookup_value_val,
            self.lookup_options_val,
            (self.can_negate and self.lookup_negated_val == "true") or self.negated,
            self.lookup_type_val,
        ]

    def choices(self, changelist):
        self.query_string = changelist.get_query_string(
            remove=self.expected_parameters()
        )
        return []

    def queryset(self, request, queryset):
        key, value, options, negated, type_ = self.value()
        if key:
            if type_ == "any" and value.isnumeric():
                filters = Q(**{f"{self.field_path}__{key}": value}) | Q(
                    **{f"{self.field_path}__{key}": int(value)}
                )
            elif type_ == "num" and value.isnumeric():
                filters = Q(**{f"{self.field_path}__{key}": float(value)})
            else:  # type_ == 'str':
                filters = Q(**{f"{self.field_path}__{key}": str(value)})

            if negated:
                if self.options and options == "e":
                    filters = ~filters
                else:
                    filters = (
                        Q(**{f"{self.field_path}__{key}__isnull": True}) | ~filters
                    )
            else:
                if options == "i":
                    filters = filters | Q(**{f"{self.field_path}__{key}__isnull": True})

            queryset = queryset.filter(filters)

        return queryset

    @property
    def media(self):
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (
            ("admin/js/vendor/select2/i18n/%s.js" % i18n_name,) if i18n_name else ()
        )
        return forms.Media(
            js=("admin/js/vendor/jquery/jquery%s.js" % extra,)
            + i18n_file
            + (
                "admin/js/jquery.init.js",
                "adminfilters/jsonfilter%s.js" % extra,
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )



import pytest

from adminfilters.utils import get_query_string


@pytest.mark.parametrize(
    "qs,new_params,remove,result",
    (
        ("", {}, [], "?"),
        ("?a=1&b=2&c=3", {}, [], "?a=1&b=2&c=3"),
        ("?a=1&b=2&c=3", {}, ["b"], "?a=1&c=3"),
        ("?a=1&b=2&c=3", {"c": 44}, [], "?a=1&b=2&c=44"),
        ("?a=1&b=2&c=3", {"c": None}, [], "?a=1&b=2"),
        ("?a=1&b=2&c=3", {"x": None}, [], "?a=1&b=2&c=3"),
        ("?a=1&b=2&c=3", None, None, "?a=1&b=2&c=3"),
    ),
)
def test_get_query_string(rf, qs, new_params, remove, result):
    request = rf.get(qs)
    ret = get_query_string(request, new_params, remove)
    assert ret == result



import sys

from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.html import strip_tags
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement

DATA = {
    "date": "2013-01-29",
    "datetime": "2013-01-01T02:18:33Z",
    "integer": 888888,
    "nullable": "bbbb",
    "time": "19:00:35",
    "bigint": 333333333,
    # "blank": "",
    "choices": 2,
    "decimal": "22.2",
    # "email": "s.apostolico@gmail.com",
    "float": 10.1,
    "generic_ip": "192.168.10.2",
    # "logic": False,
    # "not_editable": None,
    # "text": "lorem ipsum",
    # "url": "https://github.com/saxix/django-adminfilters",
    # # "flags": {},
}


def check_link_by_class(selenium, cls, view_name):
    link = selenium.find_element_by_class_name(cls)
    url = reverse(f"{view_name}")
    return f' href="{url}"' in link.get_attribute("innerHTML")


def get_all_attributes(driver, element):
    return list(
        driver.execute_script(
            "var items = {}; for (index = 0; index < arguments[0].attributes.length; ++index) {"
            " items[arguments[0].attributes[index].name] = arguments[0].attributes[index].value"
            " }; return items;",
            element,
        )
    )


def is_clickable(driver, element):
    """Tries to click the element"""
    try:
        element.click()
        return True
    except Exception:
        return False


def mykey(group, request):
    return request.META["REMOTE_ADDR"][::-1]


def callable_rate(group, request):
    if request.user.is_authenticated:
        return None
    return (0, 1)


def scroll_to(driver, *args, _timeout=10):
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.webdriver.support.ui import WebDriverWait

    wait = WebDriverWait(driver, _timeout)
    wait.until(EC.visibility_of_element_located((*args,)))
    return driver.find_element(*args)


def wait_for(driver, *args, _timeout=10):
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.webdriver.support.ui import WebDriverWait

    wait = WebDriverWait(driver, _timeout)
    wait.until(EC.visibility_of_element_located((*args,)))
    return driver.find_element(*args)


def prompt(driver, text, _timeout=10):
    from selenium.webdriver.common.alert import Alert
    from selenium.webdriver.support.expected_conditions import alert_is_present
    from selenium.webdriver.support.ui import WebDriverWait

    wait = WebDriverWait(driver, _timeout)
    wait.until(alert_is_present())

    alert = Alert(driver)
    alert.send_keys(text)
    return alert


def confirm(driver, _timeout=10):
    from selenium.webdriver.common.alert import Alert
    from selenium.webdriver.support.expected_conditions import alert_is_present
    from selenium.webdriver.support.ui import WebDriverWait

    wait = WebDriverWait(driver, _timeout)
    wait.until(alert_is_present())

    alert = Alert(driver)
    return alert


def wait_and_click(driver, *args, _timeout=10):
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.webdriver.support.ui import WebDriverWait

    wait = WebDriverWait(driver, _timeout)
    wait.until(EC.element_to_be_clickable((*args,)))
    return driver.find_element(*args).click()


def wait_for_url(driver, url):
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.webdriver.support.ui import WebDriverWait

    wait = WebDriverWait(driver, 10)
    wait.until(EC.url_contains(url))


class WebElementWrapper:
    def __init__(self, element: WebElement):
        self.element = element

    def __getattr__(self, item):
        return getattr(self.element, item)

    def __repr__(self):
        return '<{0.__module__}.{0.__name__} (session="{1}", element="{2}")>'.format(
            type(self.element), self._parent.session_id, self.element._id
        )

    @property
    def driver(self):
        return self.element.parent


class CellWrapper(WebElementWrapper):
    @cached_property
    def text(self):
        return self.element.get_attribute("innerText") or strip_tags(
            self.element.get_attribute("innerHTML")
        )


class RowWrapper(WebElementWrapper):
    @cached_property
    def values(self):
        return [e.text for e in self.cells]

    @cached_property
    def cells(self):
        ret = []
        for e in self.element.find_elements(By.CSS_SELECTOR, "td,th")[1:]:
            ret.append(CellWrapper(e))
        return ret


class EmptyChangeListWrapper(WebElementWrapper):
    rows = []
    header = []
    matrix = []

    def get_row(self, num):
        return None

    def get_col(self, num):
        return None

    def get_cell(self, row, col):
        return None


class ChangeListWrapper(EmptyChangeListWrapper):
    @classmethod
    def find_in_page(cls, driver):
        try:
            return ChangeListWrapper(
                driver.find_element(By.CSS_SELECTOR, "#changelist-form #result_list")
            )
        except NoSuchElementException:
            return EmptyChangeListWrapper(None)

    @cached_property
    def header(self) -> [RowWrapper]:
        return RowWrapper(self.element.find_element(By.CSS_SELECTOR, "thead tr"))

    @cached_property
    def rows(self) -> [RowWrapper]:
        return [
            RowWrapper(e)
            for e in self.element.find_elements(By.CSS_SELECTOR, "tbody tr")
        ]

    def get_row(self, num):
        return list(self.rows)[num]

    def get_col(self, num):
        return [row.cells[num] for row in self.rows]

    def get_cell(self, row, col):
        return self.rows[row].cells[col]

    def get_values(self, row=None, col=None):
        if row is None and col is None:
            return self.matrix
        if not (row is None or col is None):
            return self.matrix[row][col]
        if row is not None:
            return self.matrix[row]
        if col is not None:
            return [row[col] for row in self.matrix]

    @cached_property
    def matrix(self):
        cells = []
        for row in self.rows:
            cells.append([c.text for c in row.cells])
        return cells

    def pretty(self):
        from prettytable import PrettyTable

        x = PrettyTable(max_table_width=180)
        x.field_names = self.header.values
        x.add_rows(self.matrix)
        sys.stdout.write(f"\n{x}\n")


class Checkbox(WebElementWrapper):
    def checked(self, value):
        if value and not self.element.is_selected():
            self.driver.execute_script("arguments[0].click();", self.element)
        if not value and self.element.is_selected():
            self.driver.execute_script("arguments[0].click();", self.element)

    def check(self):
        if not self.element.is_selected():
            self.driver.execute_script("arguments[0].click();", self.element)

    def uncheck(self):
        if self.element.is_selected():
            self.driver.execute_script("arguments[0].click();", self.element)



import logging
from urllib import parse

from django import forms
from django.conf import settings
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.core.exceptions import FieldError, ValidationError
from django.utils.translation import get_language
from django.utils.translation import gettext as _

from .mixin import MediaDefinitionFilter, SmartListFilter
from .utils import cast_value, get_field_type, get_message_from_exception

logger = logging.getLogger(__name__)


class QueryStringFilter(MediaDefinitionFilter, SmartListFilter):
    parameter_name = "qs"
    title = "QueryString"
    template = "adminfilters/querystring.html"
    can_negate = True
    negated = False
    options = True
    separator = ","
    placeholder = _("django db lookup values")

    def __init__(self, request, params, model, model_admin):
        self.parameter_name_negated = "%s__negate" % self.parameter_name
        self._params = params
        self.lookup_field_val = self.get_parameters(self.parameter_name, pop=True)
        self.lookup_negated_val = self.get_parameters(self.parameter_name_negated, "false", pop=True)
        self.query_string = None
        self.error_message = None
        self.exception = None
        self.filters = None
        self.exclude = None
        self.model_admin = model_admin
        self.model = model
        self.validation_errors = {}
        self.filters = {}
        self.exclude = {}
        super().__init__(request, params, model, model_admin)

    @classmethod
    def factory(cls, **kwargs):
        return type("QueryStringFilter", (cls,), kwargs)

    def expected_parameters(self):
        return [self.parameter_name, self.parameter_name_negated]

    def has_output(self):
        return True

    def value(self):
        return [
            self.lookup_field_val,
            (self.can_negate and self.lookup_negated_val == "true") or self.negated,
        ]

    def choices(self, changelist):
        self.query_string = changelist.get_query_string(
            remove=self.expected_parameters()
        )
        return []

    def get_filters(self, value):
        query_params = dict(parse.parse_qsl("&".join(value.splitlines())))
        exclude = {}
        filters = {}
        for field_name, raw_value in query_params.items():
            target = filters
            cast = None

            if field_name[0] == "!":
                field_name = field_name[1:]
                target = exclude

            if field_name[0] == "#":
                field_name = field_name[1:]
                cast = int
            elif field_name[0] == ".":
                field_name = field_name[1:]
                cast = float

            field, lookup, field_type = get_field_type(self.model, field_name)
            value = cast_value(raw_value, field, lookup, force=cast)
            target[field_name] = value

        return filters, exclude

    def queryset(self, request, queryset):
        value, negated = self.value()
        if value:
            try:
                self.filters, self.exclude = self.get_filters(value)
                if not (self.filters or self.exclude):
                    self.error_message = _("Invalid django filter")
                else:
                    if negated:
                        queryset = queryset.filter(**self.exclude).exclude(
                            **self.filters
                        )
                    else:
                        queryset = queryset.filter(**self.filters).exclude(
                            **self.exclude
                        )
            except FieldError as e:
                self.error_message = get_message_from_exception(e)
            except ValidationError as e:  # pragma: no cover
                self.error_message = ", ".join(e.messages)
            except Exception as e:  # pragma: no cover
                logger.exception(e)
                if settings.DEBUG:
                    self.error_message = f"{e.__class__.__name__}: {e}"
                else:
                    logger.exception(e)
                    self.error_message = "Invalid filter"
        return queryset

    @property
    def media(self):
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (
            ("admin/js/vendor/select2/i18n/%s.js" % i18n_name,) if i18n_name else ()
        )
        return forms.Media(
            js=("admin/js/vendor/jquery/jquery%s.js" % extra,)
            + i18n_file
            + (
                "admin/js/jquery.init.js",
                "adminfilters/querystring%s.js" % extra,
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )



from django.contrib.admin import ModelAdmin
from django.contrib.admin.views.main import ChangeList
from django.contrib.auth.admin import UserAdmin

from adminfilters.autocomplete import AutoCompleteFilter
from adminfilters.combo import ChoicesFieldComboFilter
from adminfilters.depot.widget import DepotManager
from adminfilters.filters import (
    BooleanRadioFilter,
    DjangoLookupFilter,
    IntersectionFieldListFilter,
    NumberFilter,
    QueryStringFilter,
    RelatedFieldCheckBoxFilter,
    RelatedFieldRadioFilter,
    UnionFieldListFilter,
    ValueFilter,
)
from adminfilters.json import JsonFieldFilter
from adminfilters.mixin import AdminFiltersMixin
from adminfilters.value import MultiValueFilter

from .models import Artist, Band, Country


class DebugChangeList(ChangeList):
    def get_queryset(self, request):
        try:
            return super().get_queryset(request)
        except Exception as e:
            raise Exception(f"{e.__class__.__name__}: {e}")

    def get_filters(self, request):
        try:
            return super().get_filters(request)
        except Exception as e:
            raise Exception(f"{e.__class__.__name__}: {e}")


class DebugMixin:
    def get_changelist(self, request, **kwargs):
        return DebugChangeList


class DemoModelAdmin_RelatedFieldCheckBoxFilter(DebugMixin, ModelAdmin):
    list_display = [f.name for f in Artist._meta.fields]
    list_filter = (("bands", RelatedFieldCheckBoxFilter),)
    search_fields = ("name",)


class DemoModelAdmin_RelatedFieldRadioFilter(DebugMixin, ModelAdmin):
    list_display = [f.name for f in Artist._meta.fields]
    list_filter = (("bands", RelatedFieldRadioFilter),)
    search_fields = ("name",)


class DemoModelAdmin_UnionFieldListFilter(DebugMixin, ModelAdmin):
    # list_display = [f.name for f in Artist._meta.fields]
    list_display = ["name", "last_name", "full_name", "country", "year_of_birth", "active", "_bands"]
    list_filter = (("bands", UnionFieldListFilter),)
    search_fields = ("name",)

    def _bands(self, obj):
        return [b.name for b in obj.bands.all()]


class DemoModelAdmin_IntersectionFieldListFilter(DebugMixin, ModelAdmin):
    list_display = ["name", "last_name", "full_name", "country", "year_of_birth", "active", "_bands"]
    list_filter = (("bands", IntersectionFieldListFilter),)
    search_fields = ("name",)

    def _bands(self, obj):
        return [b.name for b in obj.bands.all()]


class DemoModelFieldAdmin(DebugMixin, AdminFiltersMixin, ModelAdmin):
    list_display = ("char", "integer", "logic", "email", "choices", "date")
    list_filter = (
        QueryStringFilter,
        ("choices", ChoicesFieldComboFilter),
        ("integer", NumberFilter),
    )


class CountryModelAdmin(DebugMixin, ModelAdmin):
    list_display = [f.name for f in Country._meta.fields]
    search_fields = ("name",)


class BandModelAdmin(DebugMixin, ModelAdmin):
    list_display = [f.name for f in Band._meta.fields]
    search_fields = ("name",)
    list_filter = ("genre", ChoicesFieldComboFilter), ("active", BooleanRadioFilter),


class ArtistModelAdmin(DebugMixin, AdminFiltersMixin, ModelAdmin):
    list_display = [f.name for f in Artist._meta.fields]
    list_filter = (
        DepotManager,
        ("country", AutoCompleteFilter),
        ("year_of_birth", NumberFilter),
        ("bands__name", MultiValueFilter),
        QueryStringFilter,
        DjangoLookupFilter,
        ("country__name", ValueFilter),
        ("name", MultiValueFilter),
        ("last_name", ValueFilter.factory(lookup_name="istartswith", title="LastName")),
        ("flags", JsonFieldFilter.factory(can_negate=True, options=True)),
        ("active", BooleanRadioFilter),
    )
    search_fields = ("name",)


class IUserAdmin(DebugMixin, AdminFiltersMixin, UserAdmin):
    list_display = ("username", "email", "first_name", "last_name", "is_staff")
    list_filter = (("username", ValueFilter.factory(lookup_name="istartswith")),)



from functools import partial
from unittest.mock import Mock

import pytest
from demo.models import Artist

from adminfilters.multiselect import IntersectionFieldListFilter, UnionFieldListFilter
from adminfilters.utils import get_query_string


@pytest.fixture
def fixtures(db):
    from demo.factories import ArtistFactory, BandFactory

    band1 = BandFactory(id=1, name="band1")
    band2 = BandFactory(id=2, name="band2")
    band3 = BandFactory(id=3, name="band3")
    ArtistFactory(name="a1", bands=[band1])
    ArtistFactory(name="a2", bands=[band2])
    ArtistFactory(name="a3", bands=[band3])
    ArtistFactory(name="a12", bands=[band1, band2])
    ArtistFactory(name="a23", bands=[band2, band3])


@pytest.mark.parametrize(
    "value,expected",
    [
        ("1", ["a1", "a12"]),
        ("2", ["a12", "a2", "a23"]),
        ("1,2", ["a1", "a12", "a2", "a23"]),
        ("2,3", ["a12", "a2", "a23", "a3"]),
        ("", ["a1", "a12", "a2", "a23", "a3"]),
    ],
)
def test_union(fixtures, value, expected):
    f = UnionFieldListFilter(
        Artist._meta.get_field("bands"),
        None,
        {"bands_filter": value},
        None,
        None,
        "bands",
    )
    result = f.queryset(None, Artist.objects.order_by("name"))
    value = list(result.order_by("name").values_list("name", flat=True))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        ("1", ["a1", "a12"]),
        ("2", ["a12", "a2", "a23"]),
        ("1,2", ["a12"]),
        ("2,3", ["a23"]),
        ("", ["a1", "a12", "a2", "a23", "a3"]),
    ],
)
def test_intersection(fixtures, value, expected):
    f = IntersectionFieldListFilter(
        Artist._meta.get_field("bands"),
        None,
        {"bands_filter": value},
        None,
        None,
        "bands",
    )
    result = f.queryset(None, Artist.objects.order_by("name"))
    value = list(result.order_by("name").values_list("name", flat=True))
    assert value == expected


def test_choices(fixtures):
    params = {"bands_filter": "1,2"}
    f = IntersectionFieldListFilter(
        Artist._meta.get_field("bands"), None, params, None, None, "bands"
    )
    # result = f.queryset(None, Artist.objects.order_by('name'))
    cl = Mock(
        get_query_string=partial(get_query_string, Mock(GET=params)), params=params
    )
    choices = list(f.choices(cl))
    assert len(choices) == 4
    assert choices[0] == {"display": "All", "query_string": "?", "selected": False}
    assert choices[1] == {
        "display": "band1",
        "query_string": "?bands_filter=2",
        "selected": True,
    }
    assert choices[2] == {
        "display": "band2",
        "query_string": "?bands_filter=1",
        "selected": True,
    }



from django.db import models

try:
    from django.db.models import JSONField
except ImportError:
    from django.contrib.postgres.fields import JSONField


class JSONMixin(models.Model):
    flags = JSONField(default=dict, blank=True, null=True)

    class Meta:
        abstract = True


class DemoModelField(JSONMixin, models.Model):
    char = models.CharField(max_length=255)
    integer = models.IntegerField()
    logic = models.BooleanField(default=False)
    date = models.DateField()
    datetime = models.DateTimeField()
    time = models.TimeField()
    decimal = models.DecimalField(max_digits=10, decimal_places=3)
    email = models.EmailField()
    #    filepath = models.FilePathField(path=__file__)
    float = models.FloatField()
    bigint = models.BigIntegerField()
    generic_ip = models.GenericIPAddressField()
    url = models.URLField()
    text = models.TextField()
    json = models.JSONField()
    unique = models.CharField(max_length=255, unique=True)
    nullable = models.CharField(max_length=255, null=True)
    blank = models.CharField(max_length=255, blank=True, null=True)
    not_editable = models.CharField(
        max_length=255, editable=False, blank=True, null=True
    )
    choices = models.IntegerField(
        choices=((1, "Choice 1"), (2, "Choice 2"), (3, "Choice 3"))
    )

    class Meta:
        app_label = "demo"


class Country(models.Model):
    name = models.CharField(max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "Countries"
        ordering = ("name",)


class Band(models.Model):
    name = models.CharField(max_length=255)
    genre = models.IntegerField(
        choices=(
            (1, "Rock"),
            (2, "Blues"),
            (3, "Soul"),
            (4, "Other"),
        )
    )
    active = models.BooleanField(default=True)

    def __str__(self):
        return self.name


class Artist(JSONMixin, models.Model):
    name = models.CharField(max_length=255)
    last_name = models.CharField(max_length=255)
    full_name = models.CharField(max_length=255)
    country = models.ForeignKey(Country, on_delete=models.CASCADE)
    year_of_birth = models.IntegerField()
    active = models.BooleanField(default=True)
    bands = models.ManyToManyField(Band, related_name="bands", verbose_name="Bands")

    class Meta:
        app_label = "demo"
        ordering = ("name",)

    def __str__(self):
        return self.name


#
# class DemoModel2(models.Model):
#     name = models.CharField(max_length=255)
#     demo_items = models.ForeignKey('demo.models.Artist',
#                                    related_name='items',
#                                    verbose_name='Demo Related',
#                                    on_delete=models.CASCADE)
#
#     class Meta:
#         app_label = 'demo'


class Artist_RelatedFieldCheckBoxFilter(Artist):
    class Meta:
        proxy = True
        verbose_name = "RelatedFieldCheckBoxFilter"


class Artist_RelatedFieldRadioFilter(Artist):
    class Meta:
        proxy = True
        verbose_name = "RelatedFieldRadioFilter"


class Artist_UnionFieldListFilter(Artist):
    class Meta:
        proxy = True
        verbose_name = "UnionFieldListFilter"


class Artist_IntersectionFieldListFilter(Artist):
    class Meta:
        proxy = True
        verbose_name = "IntersectionFieldListFilter"



import re

from django.contrib.admin.options import IncorrectLookupParameters

from .value import ValueFilter


class NumberFilter(ValueFilter):
    rex1 = re.compile(r"^(>=|<=|>|<|=)?([-+]?[0-9]+)$")
    re_range = re.compile(r"^(\d+)\.{2,}(\d+)$")
    re_list = re.compile(r"(\d+),?")
    re_unlike = re.compile(r"^(<>)([-+]?[0-9]+)$")
    map = {
        ">=": "gte",
        "<=": "lte",
        ">": "gt",
        "<": "lt",
        "=": "exact",
        "<>": "not",
    }
    can_negate = False

    # def __init__(self, field, request, params, model, model_admin, field_path):
    #     super().__init__(field, request, params, model, model_admin, field_path)
    #     self.lookup_kwarg = '%s__%s' % (field_path, self.lookup_name)

    @classmethod
    def factory(cls, *, title=None, **kwargs):
        if "lookup_name" in kwargs:
            raise ValueError(
                f"'lookup_name' is not a valid value for "
                f"'{cls.__class__.__name__}.factory'"
            )
        return super().factory(title=title, **kwargs)

    def placeholder(self):
        return "1 or >< <=> <> 1 or 1..10 or 1,4,5"

    # def parse_query_string(self, params):
    #     self.lookup_val = params.get(self.field.name, '')
    def expected_parameters(self):
        self.lookup_kwarg = self.field_path
        return [self.lookup_kwarg]

    def value(self):
        return [
            self.get_parameters(self.lookup_kwarg),
        ]

    def queryset(self, request, queryset):
        if self.value() and self.value()[0]:
            raw_value = self.value()[0]
            m1 = self.rex1.match(raw_value)
            m_range = self.re_range.match(raw_value)
            m_list = self.re_list.match(raw_value)
            m_unlike = self.re_unlike.match(raw_value)
            if m_unlike and m_unlike.groups():
                match = "%s__exact" % self.field_path
                op, value = self.re_unlike.match(raw_value).groups()
                queryset = queryset.exclude(**{match: value})
            else:
                if m1 and m1.groups():
                    op, value = self.rex1.match(raw_value).groups()
                    match = "%s__%s" % (self.field_path, self.map[op or "="])
                    self.filters = {match: value}
                elif m_range and m_range.groups():
                    start, end = self.re_range.match(raw_value).groups()
                    self.filters = {
                        f"{self.field_path}__gte": start,
                        f"{self.field_path}__lte": end,
                    }
                elif m_list and m_list.groups():
                    value = raw_value.split(",")
                    match = "%s__in" % self.field_path
                    self.filters = {match: value}
                # elif m_unlike and m_unlike.groups():
                #     match = '%s__exact' % self.field.name
                #     op, value = self.re_unlike.match(raw).groups()
                #     queryset = queryset.exclude(**{match: value})
                else:  # pragma: no cover
                    raise IncorrectLookupParameters()
                try:
                    queryset = queryset.filter(**self.filters)
                except Exception:
                    raise IncorrectLookupParameters(self.value())
        return queryset


# backward compatibility
MaxMinFilter = NumberFilter



from urllib.parse import urlencode

from django.core.management import BaseCommand, call_command
from django.db import IntegrityError

from adminfilters.depot.models import StoredFilter


def sample_data():
    from demo.factories import ArtistFactory, BandFactory, CountryFactory, UserFactory
    from demo.models import Artist
    from django.contrib.contenttypes.models import ContentType

    user = UserFactory(username="user")

    uk = CountryFactory(name="United Kingdom")
    australia = CountryFactory(name="Australia")

    acdc = BandFactory(name="AC/DC", active=True)
    geordie = BandFactory(name="Geordie", active=False)

    ArtistFactory(
        name="Angus",
        last_name="Young",
        full_name="Young, Angus",
        active=True,
        year_of_birth=1955,
        bands=[acdc],
        country=uk,
        flags={"v": 1},
    )

    ArtistFactory(
        name="Malcom",
        last_name="Young",
        full_name="Young, Malcom",
        year_of_birth=1953,
        active=True,
        bands=[acdc],
        country=uk,
        flags={"v": 1},
    )

    ArtistFactory(
        name="Phil",
        last_name="Rudd",
        full_name="Rudd, Phil",
        year_of_birth=1954,
        bands=[acdc],
        active=True,
        country=australia,
        flags={"full_name": "Phil Rudd"},
    )

    ArtistFactory(
        name="Brian",
        last_name="Johnson",
        full_name="Johnson, Brian",
        year_of_birth=1947,
        active=True,
        bands=[acdc, geordie],
        country=uk,
        flags={"full_name": "Brian Johnson"},
    )

    ArtistFactory(
        name="Bon",
        last_name="Scott",
        full_name="Scott, Bon",
        year_of_birth=1946,
        active=False,
        bands=[acdc],
        country=uk,
        flags={"full_name": "Bon Scott"},
    )
    ct = ContentType.objects.get_for_model(Artist)
    StoredFilter.objects.update_or_create(
        name="AC/DC",
        owner=user,
        defaults=dict(
            owner=user,
            query_string="?%s" % urlencode({"qs": "bands__name=AC/DC"}),
            content_type=ct,
        ),
    )
    StoredFilter.objects.update_or_create(
        name="QueryString",
        owner=user,
        defaults=dict(
            query_string="?%s"
            % urlencode(
                {
                    "qs": """country__name__istartswith=australia
name=Phil
year_of_birth__gt=1950
Aactive=true""",
                    "qs__negate": "false",
                }
            ),
            content_type=ct,
        ),
    )
    StoredFilter.objects.update_or_create(
        name="Active Artists",
        owner=user,
        defaults=dict(
            query_string="?%s" % urlencode({"qs": "active=true"}), content_type=ct
        ),
    )

    return [acdc, geordie]


class Command(BaseCommand):
    def handle(self, *args, **options):
        from demo.factories import ArtistFactory, DemoModelFieldFactory

        call_command("migrate")
        call_command("collectstatic", interactive=False)
        try:
            ArtistFactory.create_batch(10)
            DemoModelFieldFactory.create_batch(10)
        except IntegrityError:
            pass
        sample_data()



import pytest
from demo.utils import ChangeListWrapper
from selenium.webdriver.common.by import By

pytestmark = pytest.mark.selenium


def get_elements(selenium):
    container = selenium.find_element(By.ID, "country__exact_country__isnull")
    el = selenium.find_elements(By.CSS_SELECTOR, "input.select2-search__field")
    return [
        container,
        container.find_element(By.CSS_SELECTOR, "span[role=combobox]"),
        el[0] if el else None,
        ChangeListWrapper.find_in_page(selenium),
    ]


@pytest.mark.selenium
def test_autocomplete(admin_site):
    __, select2, *__ = get_elements(admin_site.driver)
    select2.click()
    __, __, input_text, __ = get_elements(admin_site.driver)
    input_text.send_keys("United K")
    admin_site.wait_and_click(By.XPATH, '//li[contains(text(), "United Kingdom")]')
    *__, cl = get_elements(admin_site.driver)
    assert set(cl.get_values(None, 5)) == {"United Kingdom"}



from unittest.mock import patch

import pytest
from demo.admin import ArtistModelAdmin
from demo.models import Artist, Band
from django.contrib.admin import ModelAdmin, site

from adminfilters.checkbox import RelatedFieldCheckBoxFilter
from adminfilters.checks import check_adminfilters_media
from adminfilters.filters import ValueFilter
from adminfilters.querystring import QueryStringFilter


def test():
    ret = check_adminfilters_media(None)
    assert ret == [], [e.msg for e in ret]


@pytest.mark.parametrize("filter_class", [QueryStringFilter, ("name", ValueFilter)])
def test_check_error(filter_class):
    class InvalidModelAdmin(ModelAdmin):
        list_filter = (
            "active",
            filter_class,
            ("genre", RelatedFieldCheckBoxFilter),
        )

    with patch.dict(
        site._registry, {Artist: ArtistModelAdmin, Band: InvalidModelAdmin}, clear=True
    ):
        ret = check_adminfilters_media(None)
        assert len(ret) == 1, [e.msg for e in ret]



from django.contrib.admin.filters import AllValuesFieldListFilter, ChoicesFieldListFilter, RelatedFieldListFilter

from .mixin import WrappperMixin


class AllValuesComboFilter(WrappperMixin, AllValuesFieldListFilter):
    template = "adminfilters/combobox.html"


class RelatedFieldComboFilter(WrappperMixin, RelatedFieldListFilter):
    template = "adminfilters/combobox.html"


class ChoicesFieldComboFilter(WrappperMixin, ChoicesFieldListFilter):
    template = "adminfilters/combobox.html"



import random

import factory.fuzzy
import faker
from django.contrib.auth.models import User
from factory.base import FactoryMetaClass

from . import models

factories_registry = {}


class AutoRegisterFactoryMetaClass(FactoryMetaClass):
    def __new__(mcs, class_name, bases, attrs):
        new_class = super().__new__(mcs, class_name, bases, attrs)
        factories_registry[new_class._meta.model] = new_class
        return new_class


class ModelFactory(
    factory.django.DjangoModelFactory, metaclass=AutoRegisterFactoryMetaClass
):
    pass


def get_flags():
    # Convert to plain ascii text
    f = lambda: random.choice(
        [
            ("int", random.randint(1, 100)),
            ("chr", chr(random.randrange(65, 90))),
            ("int", "__"),
            ("chr", ""),
            ("chr", None),
            ("int", None),
            ("chr", "__"),
        ]
    )
    value = f()
    if value[1] == "__":
        base = {}
    else:
        base = dict([value])
    value = f()
    if value[1] != "__":
        base.update(dict([value]))
    return base


class DemoModelFieldFactory(ModelFactory):
    char = factory.Faker("name")
    integer = factory.Faker("pyint")
    date = factory.Faker("date_this_decade")
    datetime = factory.Faker("date_this_decade")
    time = factory.Faker("time")
    logic = factory.Faker("pybool")
    decimal = factory.Faker("coordinate")
    float = factory.Faker("pyfloat")
    bigint = factory.Faker("pyint")
    generic_ip = factory.Faker("ipv4")
    choices = factory.fuzzy.FuzzyChoice([1, 2, 3])
    unique = factory.Sequence(lambda a: a)
    email = factory.Faker("email")
    json = {"char": "string", "integer": 100, "float": 2.0}

    class Meta:
        model = models.DemoModelField

    @factory.lazy_attribute
    def flags(self):
        return get_flags()


class UserFactory(ModelFactory):
    class Meta:
        model = User
        django_get_or_create = ("username",)


class CountryFactory(ModelFactory):
    name = factory.Faker("country")

    class Meta:
        model = models.Country
        django_get_or_create = ("name",)


class BandFactory(ModelFactory):
    name = factory.Faker("name")
    genre = factory.fuzzy.FuzzyChoice([1, 2, 3, 4])

    class Meta:
        model = models.Band
        django_get_or_create = ("name",)


class ArtistFactory(ModelFactory):
    name = factory.Faker("first_name")
    last_name = factory.Sequence(lambda n: f'Dummy{faker.Faker().unique.last_name()}{n}')
    full_name = factory.LazyAttribute(lambda o: f"{o.last_name}, {o.name}")

    country = factory.SubFactory(CountryFactory)
    year_of_birth = factory.Faker("year")

    class Meta:
        model = models.Artist
        django_get_or_create = ("name", "last_name")

    @factory.post_generation
    def bands(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted is None:
            # A list of groups were passed in, use them
            extracted = [BandFactory()]
        for topping in extracted:
            self.bands.add(topping)

    @factory.lazy_attribute
    def flags(self):
        return get_flags()


def get_factory_for_model(_model):
    class Meta:
        model = _model

    if _model in factories_registry:
        return factories_registry[_model]
    return type("AAA", (ModelFactory,), {"Meta": Meta})



import json

from django import forms
from django.conf import settings
from django.contrib import messages
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.utils.translation import get_language
from django.utils.translation import gettext as _

from adminfilters.mixin import MediaDefinitionFilter, SmartFieldListFilter


class ValueFilter(MediaDefinitionFilter, SmartFieldListFilter):
    template = "adminfilters/value.html"
    toggleable = False
    filter_title = None
    lookup_name = "exact"
    button = True
    can_negate = True
    negated = False

    def __init__(self, field, request, params, model, model_admin, field_path):
        # self.lookup_negated_val = None
        # self.model = model
        # self.field = field
        # self.lookup_val = None
        # self.lookup_kwarg = '%s__%s' % (field_path, self.lookup_name)
        # self.lookup_kwarg_negated = '%s__negate' % self.lookup_kwarg
        # self.parse_query_string(params)
        self.lookup_kwarg = None
        self.lookup_kwarg_negated = None
        self.field_path = field_path
        self.parameters = {}
        self.filters = {}
        for p in self.expected_parameters():
            if p in params:
                self.parameters[p] = params.pop(p)

        super().__init__(field, request, params, model, model_admin, field_path)
        self._params = self.parameters
        self.title = self._get_title()
        # self.query_string = get_query_string(request, remove=self.expected_parameters())

        # self.params = params
        # self.query_values = []
        # self.operator = '+'

    def expected_parameters(self):
        self.lookup_kwarg = "%s__%s" % (self.field_path, self.lookup_name)
        self.lookup_kwarg_negated = "%s__negate" % self.lookup_kwarg
        return [self.lookup_kwarg, self.lookup_kwarg_negated]

    def value(self):
        return [
            self.get_parameters(self.lookup_kwarg),
            self.get_parameters(self.lookup_kwarg_negated) == "true",
            # self.parameters[self.lookup_kwarg],
            # self.parameters[self.lookup_kwarg_negated] == 'true'
            # self.lookup_val,
            # self.lookup_negated_val == 'true'
        ]

    def js_options(self):
        return json.dumps(
            dict(button=self.button, canNegate=self.can_negate, negated=self.negated)
        )

    def _get_title(self):
        if self.filter_title:
            return self.filter_title
        elif "__" in self.field_path:
            return self.field_path.replace("__", "->")
        return getattr(self.field, "verbose_name", self.field_path)

    @classmethod
    def factory(cls, *, title=None, lookup_name="exact", **kwargs):
        kwargs["filter_title"] = title
        kwargs["lookup_name"] = lookup_name
        return type("ValueFilter", (cls,), kwargs)

    # def parse_query_string(self, params):
    #     self.lookup_negated_val = params.get(self.lookup_kwarg_negated)
    #     self.lookup_val = params.get(self.lookup_kwarg, '')

    def queryset(self, request, queryset):
        target, exclude = self.value()
        if target:
            try:
                self.filters = {self.lookup_kwarg: target}
                if exclude:
                    queryset = queryset.exclude(**self.filters)
                else:
                    queryset = queryset.filter(**self.filters)
            except Exception as e:
                msg = _("%s filter ignored due to an error %s") % (self.title, e)
                self.model_admin.message_user(request, msg, messages.ERROR)
                pass
        return queryset

    def choices(self, changelist):
        self.query_string = changelist.get_query_string(
            remove=self.expected_parameters()
        )
        return []

    @property
    def media(self):
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (
            ("admin/js/vendor/select2/i18n/%s.js" % i18n_name,) if i18n_name else ()
        )
        return forms.Media(
            js=("admin/js/vendor/jquery/jquery%s.js" % extra,)
            + i18n_file
            + (
                "admin/js/jquery.init.js",
                "adminfilters/value%s.js" % extra,
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )


class MultiValueFilter(ValueFilter):
    template = "adminfilters/value_multi.html"
    separator = ","
    filter_title = None
    lookup_name = "in"

    def placeholder(self):
        return _("comma separated list of values")

    def value(self):
        values = self.get_parameters(self.lookup_kwarg, None, multi=True)
        return [values, self.get_parameters(self.lookup_kwarg_negated, "") == "true"]


TextFieldFilter = ValueFilter
ForeignKeyFieldFilter = TextFieldFilter
MultiValueTextFieldFilter = MultiValueFilter



import pytest
from demo.admin import ArtistModelAdmin
from demo.models import Artist, Country
from demo.urls import public_site

from adminfilters.autocomplete import AutoCompleteFilter
from adminfilters.compat import DJANGO_MAJOR


@pytest.fixture
def fixtures(db):
    from demo.factories import ArtistFactory

    ArtistFactory(name="a1", country__name="c1")
    ArtistFactory(name="a2", country__name="c1")
    ArtistFactory(name="b1", country__name="c2")
    ArtistFactory(name="c1", country__name="c2")


@pytest.mark.parametrize("value", ["c1", "c2"])
def test_filter(fixtures, value):
    country = Country.objects.get(name=value)
    if DJANGO_MAJOR < 5:
        country_pk = country.pk
    else:
        country_pk = [country.pk]
    f = AutoCompleteFilter(
        Artist._meta.get_field("country"),
        None,
        {"country__exact": country_pk, "name__isnull": ""},
        Artist,
        public_site._registry[Artist],
        "country",
    )
    qs = f.queryset(None, Artist.objects.all())
    result = set(qs.values_list("country__name", flat=True))
    assert result == {value}


def test_media():
    assert AutoCompleteFilter.factory(title="Title")(
        None, None, {}, Artist, ArtistModelAdmin(Artist, public_site), "last_name"
    ).media


def test_url():
    assert AutoCompleteFilter.factory(title="Title")(
        None, None, {}, Artist, ArtistModelAdmin(Artist, public_site), "last_name"
    ).get_url() == '/autocomplete/'



import contextlib
import logging
from urllib.parse import urljoin

import pytest
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By


@pytest.fixture
def chrome_options(request):
    chrome_options = Options()
    if not request.config.getvalue("show_browser"):
        chrome_options.add_argument("--headless")
    chrome_options.add_argument("--no-sandbox")
    chrome_options.add_argument("--disable-dev-shm-usage")
    chrome_options.add_argument("--allow-insecure-localhost")
    chrome_options.add_argument("--disable-translate")
    chrome_options.add_argument("--disable-extensions")
    chrome_options.add_argument("--start-maximized")
    chrome_options.add_argument("--lang=en-GB")
    chrome_options.add_argument("--disable-gpu")
    chrome_options.add_argument("--disable-browser-side-navigation")
    chrome_options.add_argument("--ignore-certificate-errors")
    chrome_options.add_argument("--proxy-server='direct://'")
    chrome_options.add_argument("--proxy-bypass-list=*")
    return chrome_options


SELENIUM_DEFAULT_PAGE_LOAD_TIMEOUT = 3
SELENIUM_DEFAULT_IMPLICITLY_WAIT = 2
SELENIUM_DEFAULT_SCRIPT_TIMEOUT = 2


@contextlib.contextmanager
def timeouts(driver, wait=None, page=None, script=None):
    from selenium.webdriver.common.timeouts import Timeouts

    _current: Timeouts = driver.timeouts
    if wait:
        driver.implicitly_wait(wait)
    if page:
        driver.set_page_load_timeout(page)
    if script:
        driver.set_script_timeout(script)
    yield
    driver.timeouts = _current


def set_input_value(driver, *args):
    rules = args[:-1]
    el = driver.find_element(*rules)
    el.clear()
    el.send_keys(args[-1])


def get_errors(driver):
    """
    Checks browser for errors, returns a list of errors
    :param driver:
    :return:
    """
    try:
        browser_logs = driver.get_log("browser")
    except (ValueError, WebDriverException) as e:
        # Some browsers does not support getting logs
        logging.debug(
            "Could not get browser logs for driver %s due to exception: %s", driver, e
        )
        return []

    errors = [entry for entry in browser_logs if entry["level"] == "SEVERE"]

    return errors


@pytest.fixture
def selenium(driver):
    from demo.utils import confirm, prompt, wait_and_click, wait_for

    driver.with_timeouts = timeouts.__get__(driver)
    driver.set_input_value = set_input_value.__get__(driver)
    driver.wait_for = wait_for.__get__(driver)
    driver.wait_and_click = wait_and_click.__get__(driver)
    driver.prompt = prompt.__get__(driver)
    driver.confirm = confirm.__get__(driver)
    driver.get_errors = get_errors.__get__(driver)
    yield driver


@pytest.fixture(autouse=True)
def data():
    from demo.factories import ArtistFactory
    from demo.management.commands.init_demo import sample_data

    batch = ArtistFactory.create_batch(20)
    sample_data()
    return batch


class AdminSite:
    def __init__(self, live_server, driver):
        self.live_server = live_server
        self.driver = driver

    def open(self, path=""):
        self.driver.get(urljoin(self.live_server.url, path))
        dim = self.driver.get_window_size()
        self.driver.set_window_size(1100, dim["height"])

    def wait_for(self, *args):
        return self.driver.wait_for(*args)

    def prompt(self, *args):
        return self.driver.prompt(*args)

    def confirm(self, *args):
        return self.driver.confirm(*args)

    def wait_and_click(self, *args):
        return self.driver.wait_and_click(*args)

    def get_errors(self):
        return self.driver.get_errors()


@pytest.fixture
def admin_site(live_server, selenium, data):
    site = AdminSite(live_server, selenium)
    site.open("/")
    site.wait_for(By.LINK_TEXT, "Artists").click()
    errors = [err for err in site.get_errors() if err and 'favicon' not in err['message']]
    assert len(errors) == 0, "\n".join(["{message}".format(**err) for err in errors])
    return site



import pytest
from demo.utils import ChangeListWrapper, Checkbox
from selenium.webdriver.common.by import By

pytestmark = pytest.mark.selenium


def get_elements(selenium):
    container = selenium.find_element(
        By.ID, "last_name__istartswith_last_name__istartswith__negate"
    )
    return [
        container.find_element(By.CSS_SELECTOR, "input[type=text]"),
        Checkbox(container.find_element(By.CSS_SELECTOR, "input[type=checkbox]")),
        container.find_element(By.CSS_SELECTOR, "a.button"),
        ChangeListWrapper.find_in_page(selenium),
    ]


@pytest.mark.selenium
def test_value_simple_filter(data, admin_site):
    input_text, negate, button, cl = get_elements(admin_site.driver)
    input_text.send_keys("Young")
    button.click()
    input_text, negate, button, cl = get_elements(admin_site.driver)
    assert set(cl.get_values(None, 3)) == {"Young"}
    negate.click()
    button.click()
    input_text, negate, button, cl = get_elements(admin_site.driver)
    assert "Young" not in set(cl.get_values(None, 3))


@pytest.mark.selenium
def test_multivalue_simple_filter(data, admin_site):
    input_text, negate, button, cl = get_elements(admin_site.driver)
    input_text.send_keys("Young")
    button.click()
    input_text, negate, button, cl = get_elements(admin_site.driver)
    assert set(cl.get_values(None, 3)) == {"Young"}

    negate.click()
    button.click()
    input_text, negate, button, cl = get_elements(admin_site.driver)
    assert "Young" not in set(cl.get_values(None, 3))


@pytest.mark.selenium
def test_multivalue_filter_clear(data, admin_site):
    input_text, negate, button, cl = get_elements(admin_site.driver)
    input_text.send_keys("Young")
    button.click()
    input_text, negate, button, cl = get_elements(admin_site.driver)
    assert set(cl.get_values(None, 3)) == {"Young"}
    assert len(cl.rows) == 2

    button.click()
    input_text, negate, button, cl = get_elements(admin_site.driver)
    assert len(cl.rows) == 25



from django import forms
from django.conf import settings
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.core.exceptions import FieldError, ValidationError
from django.db.models import Q
from django.utils.translation import get_language
from django.utils.translation import gettext as _

from .mixin import MediaDefinitionFilter, SmartListFilter
from .utils import cast_value, get_field_type, get_message_from_exception


class DjangoLookupFilter(MediaDefinitionFilter, SmartListFilter):
    parameter_name = "dj"
    title = "Django Lookup"
    template = "adminfilters/dj.html"
    can_negate = True
    negated = False
    button = True
    field_placeholder = _("field lookup. Es. name__startswith")
    placeholder = _("field value")

    def __init__(self, request, params, model, model_admin):
        self.lookup_kwarg_key = "%s__key" % self.parameter_name
        self.lookup_kwarg_value = "%s__value" % self.parameter_name
        self.lookup_kwarg_negated = "%s__negate" % self.parameter_name
        self._params = params
        self.lookup_field_val = self.get_parameters(self.lookup_kwarg_key, pop=True)
        self.lookup_value_val = self.get_parameters(self.lookup_kwarg_value, pop=True)
        self.lookup_negated_val = self.get_parameters(self.lookup_kwarg_negated, "false", pop=True)
        self.error_message = None
        self.exception = None
        self.filters = None
        self.exclude = None
        self.model = model
        self.query_string = None
        super().__init__(request, params, model, model_admin)

    @classmethod
    def factory(cls, **kwargs):
        return type("DjangoLookupFilter", (cls,), kwargs)

    def expected_parameters(self):
        return [
            self.lookup_kwarg_key,
            self.lookup_kwarg_value,
            self.lookup_kwarg_negated,
        ]

    def has_output(self):
        return True

    def value(self):
        try:
            if self.lookup_field_val:
                field, lookup, field_type = get_field_type(
                    self.model, self.lookup_field_val
                )
                value = cast_value(self.lookup_value_val, field, lookup)
                return [
                    self.lookup_field_val,
                    value,
                    (self.can_negate and self.lookup_negated_val == "true")
                    or self.negated,
                ]
        except ValidationError as e:  # pragma: no cover
            self.exception = e
            self.error_message = str(e.message)

        return [None, None, None]

    def choices(self, changelist):
        self.query_string = changelist.get_query_string(
            remove=self.expected_parameters()
        )
        return []

    def queryset(self, request, queryset):
        try:
            key, value, negated = self.value()
            if key:
                try:
                    self.filters = Q(**{f"{self.lookup_field_val}": value})

                    if negated:
                        self.filters = ~self.filters

                    queryset = queryset.filter(self.filters)
                except FieldError as e:
                    self.exception = e
                    self.error_message = get_message_from_exception(e)
        except ValidationError as e:  # pragma: no cover
            self.exception = e
            self.error_message = str(e.message)

        return queryset

    @property
    def media(self):
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (
            ("admin/js/vendor/select2/i18n/%s.js" % i18n_name,) if i18n_name else ()
        )
        return forms.Media(
            js=("admin/js/vendor/jquery/jquery%s.js" % extra,)
            + i18n_file
            + (
                "admin/js/jquery.init.js",
                "adminfilters/dj%s.js" % extra,
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )



import pytest
from demo.factories import ArtistFactory
from demo.models import Artist

from adminfilters.json import JsonFieldFilter


@pytest.fixture
def fixtures(db):
    ArtistFactory(flags={"v": 1})
    ArtistFactory(flags={"v": 1})
    ArtistFactory(flags={"v": 2})
    ArtistFactory(flags={"v": "2"})
    ArtistFactory(flags={})


def test_JsonFieldFilter(fixtures):
    f = JsonFieldFilter(
        "json",
        None,
        {
            "flags__key": "v",
            "flags__value": "1",
            "flags__negate": "false",
            "flags__options": "e",
        },
        None,
        None,
        "flags",
    )
    result = f.queryset(None, Artist.objects.all())
    assert list(result.order_by("flags__v").values_list("flags__v", flat=True)) == [
        1,
        1,
    ]

    f = JsonFieldFilter(
        "json",
        None,
        {
            "flags__key": "v",
            "flags__value": "1",
            "flags__negate": "false",
            "flags__options": "i",
        },
        None,
        None,
        "flags",
    )
    result = f.queryset(None, Artist.objects.all())
    assert list(result.order_by("flags__v").values_list("flags__v", flat=True)) == [
        1,
        1,
        None,
    ]
    # negate
    f = JsonFieldFilter(
        "json",
        None,
        {
            "flags__key": "v",
            "flags__value": "1",
            "flags__negate": "true",
            "flags__options": "e",
        },
        None,
        None,
        "flags",
    )
    result = f.queryset(None, Artist.objects.all())
    # this is a Django bug. It should returns [2, "2"]
    assert list(result.order_by("flags__v").values_list("flags__v", flat=True)) == [
        "2",
        2,
    ]

    # negate / include
    f = JsonFieldFilter(
        "json",
        None,
        {
            "flags__key": "v",
            "flags__value": "1",
            "flags__negate": "true",
            "flags__options": "i",
        },
        None,
        None,
        "flags",
    )
    result = f.queryset(None, Artist.objects.all())
    # this is a Django bug. It should returns [2, "2"]
    assert list(result.order_by("flags__v").values_list("flags__v", flat=True)) == [
        "2",
        2,
        None,
    ]

    # cast to int
    f = JsonFieldFilter(
        "json",
        None,
        {
            "flags__key": "v",
            "flags__value": "2",
            "flags__negate": "false",
            "flags__type": "num",
            "flags__options": "e",
        },
        None,
        None,
        "flags",
    )
    result = f.queryset(None, Artist.objects.all())
    assert list(result.order_by("flags__v").values_list("flags__v", flat=True)) == [2]

    # cast to char
    f = JsonFieldFilter(
        "json",
        None,
        {
            "flags__key": "v",
            "flags__value": "2",
            "flags__negate": "false",
            "flags__type": "str",
            "flags__options": "e",
        },
        None,
        None,
        "flags",
    )
    result = f.queryset(None, Artist.objects.all())
    assert list(result.order_by("flags__v").values_list("flags__v", flat=True)) == ["2"]

    # cast to char/include
    f = JsonFieldFilter(
        "json",
        None,
        {
            "flags__key": "v",
            "flags__value": "2",
            "flags__negate": "false",
            "flags__type": "str",
            "flags__options": "i",
        },
        None,
        None,
        "flags",
    )
    result = f.queryset(None, Artist.objects.all())
    assert list(result.order_by("flags__v").values_list("flags__v", flat=True)) == [
        "2",
        None,
    ]



from django.contrib.admin.filters import RelatedFieldListFilter
from django.db.models.fields.related import ForeignObjectRel
from django.db.models.query_utils import Q
from django.utils.encoding import smart_str
from django.utils.translation import gettext as _

from .mixin import MediaDefinitionFilter, WrappperMixin
from .utils import parse_bool


class RelatedFieldCheckBoxFilter(
    WrappperMixin, MediaDefinitionFilter, RelatedFieldListFilter
):
    template = "adminfilters/checkbox.html"

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.model_admin = model_admin
        super().__init__(field, request, params, model, model_admin, field_path)
        self.lookup_kwarg = "%s__%s" % (field_path, field.target_field.name)
        self.lookup_val = request.GET.getlist(self.lookup_kwarg, [])

    def queryset(self, request, queryset):
        filters = Q()
        if self.lookup_val:
            filters.add(Q(**{f"{self.lookup_kwarg}__in": self.lookup_val}), Q.OR)

        if self.lookup_val_isnull:
            filters.add(
                Q(**{self.lookup_kwarg_isnull: parse_bool(self.lookup_val_isnull)}),
                Q.OR,
            )
        return queryset.filter(filters)

    def choices(self, cl):
        try:
            from django.contrib.admin.views.main import EMPTY_CHANGELIST_VALUE
        except ImportError:
            EMPTY_CHANGELIST_VALUE = self.model_admin.get_empty_value_display()

        uncheck_all = []
        uncheck_all.append("{}={}".format(self.lookup_kwarg_isnull, 1))
        for i in self.lookup_choices:
            uncheck_all.append("{}={}".format(self.lookup_kwarg, i[0]))

        yield {
            "selected": not len(self.lookup_val) and not self.lookup_val_isnull,
            "query_string": cl.get_query_string(
                {}, [self.lookup_kwarg, self.lookup_kwarg_isnull]
            ),
            "display": _("All"),
            "check_to_remove": "&".join(uncheck_all),
        }
        yield {
            "selected": self.lookup_val_isnull,
            "query_string": cl.get_query_string(
                {self.lookup_kwarg_isnull: 1},
                [self.lookup_kwarg, self.lookup_kwarg_isnull],
            ),
            "display": _("None"),
            "uncheck_to_remove": "{}=1".format(self.lookup_kwarg_isnull),
        }
        for pk_val, val in self.lookup_choices:
            yield {
                "selected": smart_str(pk_val) in self.lookup_val,
                "query_string": cl.get_query_string(
                    {
                        self.lookup_kwarg: pk_val,
                    },
                    [self.lookup_kwarg_isnull],
                ),
                "display": val,
                "uncheck_to_remove": "{}={}".format(self.lookup_kwarg, pk_val)
                if pk_val
                else "",
            }
        if (
            isinstance(self.field, ForeignObjectRel)
            and self.field.field.null
            or hasattr(self.field, "rel")
            and self.field.null
        ):
            yield {
                "selected": bool(self.lookup_val_isnull),
                "query_string": cl.get_query_string(
                    {
                        self.lookup_kwarg_isnull: "True",
                    },
                    [self.lookup_kwarg],
                ),
                "uncheck_to_remove": "{}=1".format(self.lookup_kwarg_isnull),
                "display": EMPTY_CHANGELIST_VALUE,
            }



from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import Permission


class AnonymousAccessUserBackend(ModelBackend):
    supports_object_permissions = False
    supports_anonymous_user = True

    def get_all_permissions(self, user_obj, obj=None):
        return (
            Permission.objects.all()
            .values_list("content_type__app_label", "codename")
            .order_by()
        )

    def get_group_permissions(self, user_obj, obj=None):
        return (
            Permission.objects.all()
            .values_list("content_type__app_label", "codename")
            .order_by()
        )

    def has_perm(self, user_obj, perm, obj=None):
        return True

    def has_module_perms(self, user_obj, app_label):
        return True


#
# class AnyUserAuthBackend(ModelBackend):
#     def authenticate(self, request, username=None, password=None, **kwargs):
#         user, __ = User.objects.update_or_create(username=username,
#                                                  defaults=dict(is_staff=True,
#                                                  is_active=True,
#                                                  is_superuser=True,
#                                                  email=f'{username}@demo.org'))
#         return user



#!/usr/bin/env python
import ast
import codecs
import os
import re

from setuptools import find_packages, setup

ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__)))
init = os.path.join(ROOT, 'src', 'adminfilters', '__init__.py')

_version_re = re.compile(r'__version__\s+=\s+(.*)')
_name_re = re.compile(r'NAME\s+=\s+(.*)')

with open(init, 'rb') as f:
    content = f.read().decode('utf-8')
    version = str(ast.literal_eval(_version_re.search(content).group(1)))
    name = str(ast.literal_eval(_name_re.search(content).group(1)))


def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    return codecs.open(os.path.join(here, 'src', 'requirements', *parts), 'r').read()


install_requires = read('install.pip')
tests_requires = read('testing.pip')
dev_requires = tests_requires + read('develop.pip')

setup(
    name=name,
    version=version,
    url='https://github.com/saxix/django-adminfilters',
    download_url='https://github.com/saxix/django-adminfilters',
    author='sax',
    author_email='s.apostolico@gmail.com',
    description='Extra filters for django admin site',
    license='MIT',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    extras_require={'test': tests_requires, 'dev': dev_requires},
    platforms=['any'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 5.0',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Intended Audience :: Developers',
    ],
    long_description=codecs.open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
)



from django.contrib.admin.filters import (
    AllValuesFieldListFilter,
    BooleanFieldListFilter,
    ChoicesFieldListFilter,
    RelatedFieldListFilter,
)

from .mixin import WrappperMixin


class AllValuesRadioFilter(WrappperMixin, AllValuesFieldListFilter):
    template = "adminfilters/radio.html"


class RelatedFieldRadioFilter(WrappperMixin, RelatedFieldListFilter):
    template = "adminfilters/radio.html"


class ChoicesFieldRadioFilter(WrappperMixin, ChoicesFieldListFilter):
    template = "adminfilters/radio.html"


class BooleanRadioFilter(WrappperMixin, BooleanFieldListFilter):
    template = "adminfilters/radio.html"



from inspect import isclass

from django.contrib.admin import ListFilter
from django.core.checks import Error, register

from adminfilters.mixin import AdminFiltersMixin, MediaDefinitionFilter


@register()
def check_adminfilters_media(*args, **kwargs):
    errors = []
    try:
        from django.contrib.admin import site

        for model, model_admin in site._registry.items():
            if isclass(model_admin) and not issubclass(model_admin, AdminFiltersMixin):
                for filter_fields in model_admin.list_filter:
                    filter_class = None
                    if isinstance(filter_fields, tuple):
                        filter_class = filter_fields[1]
                    elif isclass(filter_fields) and issubclass(
                        filter_fields, (ListFilter,)
                    ):
                        filter_class = filter_fields

                    if filter_class and issubclass(filter_class, MediaDefinitionFilter):
                        errors.append(
                            Error(
                                f"To use '{filter_class.__name__}', "
                                f"'{model_admin.__name__}' must inherit from 'AdminFiltersMixin'"
                            )
                        )
                        break
    except Exception:  # pragma: no-cover
        raise
    return errors



import pytest
from demo.utils import ChangeListWrapper
from selenium.webdriver.common.by import By
from selenium.webdriver.support.select import Select

from adminfilters.depot.models import StoredFilter

pytestmark = pytest.mark.selenium


def get_elements(selenium):
    container = selenium.find_element(By.ID, "adminfilters_depot_adminfilters_depot_op")
    btn = container.find_elements(By.CSS_SELECTOR, "a.button")
    return [
        container,
        Select(container.find_element(By.CSS_SELECTOR, "select")),
        btn[0] if btn else None,
        ChangeListWrapper.find_in_page(selenium),
    ]


@pytest.mark.selenium
def test_save_filter(admin_site):
    qs = "?name__in=Angus&name__in__negate=false"
    admin_site.open(f"/demo/artist/{qs}")
    __, select, button, cl = get_elements(admin_site.driver)

    button.click()
    admin_site.prompt("Filter #1").accept()

    __, select, button, cl = get_elements(admin_site.driver)
    sf = StoredFilter.objects.get(name="Filter #1")
    assert sf.query_string == qs


@pytest.mark.selenium
def test_select_filter(admin_site):
    sf = StoredFilter.objects.get(name="QueryString")

    __, select, button, cl = get_elements(admin_site.driver)
    select.select_by_visible_text(sf.name)
    admin_site.wait_for(By.TAG_NAME, "body")
    assert sf.query_string in admin_site.driver.current_url


@pytest.mark.selenium
def test_delete_filter(admin_site):
    sf = StoredFilter.objects.get(name="QueryString")
    __, select, button, cl = get_elements(admin_site.driver)
    select.select_by_visible_text(sf.name)
    admin_site.wait_for(By.TAG_NAME, "body")
    __, select, button, cl = get_elements(admin_site.driver)
    button.click()
    admin_site.confirm().accept()
    admin_site.wait_for(By.TAG_NAME, "body")
    assert not StoredFilter.objects.filter(name="QueryString").exists()



import pytest
from demo.utils import ChangeListWrapper, Checkbox
from selenium.webdriver.common.by import By

pytestmark = pytest.mark.selenium


def get_elements(selenium):
    container = selenium.find_element(By.ID, "qs_qs__negate")
    return [
        container,
        container.find_element(By.CSS_SELECTOR, "textarea"),
        container.find_element(By.CSS_SELECTOR, "a.button"),
        Checkbox(container.find_element(By.CSS_SELECTOR, "input[type=checkbox]")),
        ChangeListWrapper.find_in_page(selenium),
    ]


@pytest.mark.parametrize(
    "query,negated,check",
    [
        ("name=Angus", False, lambda cl: len(cl.rows) == 1),
        ("bands__name=AC/DC", False, lambda cl: len(cl.rows) == 5),
        (
            "bands__name=AC/DC",
            True,
            lambda cl: "Rudd, Phil" not in cl.get_values(None, 4),
        ),
        (
            "country__name=United Kingdom\nbands__name=AC/DC\n!name=Angus",
            False,
            lambda cl: "Young, Angus" not in cl.get_values(None, 4),
        ),
        (
            "bands__name=AC/DC\nname__in=Angus,Malcom",
            False,
            lambda cl: {"Young"} == set(cl.get_values(None, 3)),
        ),
    ],
)
@pytest.mark.selenium
def test_querystring(admin_site, query, negated, check):
    __, textarea, button, negate, cl = get_elements(admin_site.driver)
    textarea.send_keys(query)
    negate.checked(negated)
    button.click()
    __, textarea, button, negate, cl = get_elements(admin_site.driver)
    assert check(cl), cl.pretty()


@pytest.mark.selenium
@pytest.mark.parametrize(
    "flag,results",
    [
        ("active=true", 1),
        ("active=false", 0),
        ("!active=false", 1),
        ("!active=true", 0),
    ],
)
def test_querystring_bool(admin_site, flag, results):
    __, textarea, button, negate, cl = get_elements(admin_site.driver)

    negate.uncheck()
    textarea.clear()
    textarea.send_keys(f"{flag}\ncountry__name=Australia\nname=Phil")
    button.click()
    __, textarea, button, negate, cl = get_elements(admin_site.driver)
    assert len(cl.rows) == results


@pytest.mark.selenium
def test_errored_querystring(admin_site):
    __, textarea, button, negate, cl = get_elements(admin_site.driver)

    textarea.send_keys("invalid_field=AAAA")
    button.click()
    assert "Unknown field 'invalid_field'" in admin_site.driver.page_source


@pytest.mark.selenium
def test_invalid_querystring(admin_site):
    __, textarea, button, negate, cl = get_elements(admin_site.driver)

    textarea.send_keys("=")
    button.click()
    assert "Invalid django filter" in admin_site.driver.page_source



import pytest
from demo.models import DemoModelField

from adminfilters.querystring import QueryStringFilter


@pytest.fixture
def fixtures(db):
    from demo.utils import DATA

    for i in range(1, 5):
        values = DATA.copy()
        values["unique"] = i
        values["logic"] = bool(i % 2)
        values["json"] = {
            "char": chr(ord("a") + i),
            "integer": i * 100,
            "float": i * 1.0,
        }

        DemoModelField.objects.create(**values)


@pytest.mark.parametrize(
    "op,expected,error",
    [
        ("unique=1", "1", None),
        ("unique__in=1,2,3", "1,2,3", None),
        ("!unique=2", "1,3,4", None),
        ("logic=true", "1,3", None),
        ("logic=false", "2,4", None),
        ("!logic=false", "1,3", None),
        ("json__char=b", "1", None),
        ("#json__integer=100", "1", None),
        ("!#json__integer=100", "2,3,4", None),
        (".json__float=1.0", "1", None),
        ("logic__in=true,false", "1,2,3,4", None),
        ("wrong=1", "1,2,3,4", "Unknown field 'wrong'"),
        ("logic__x=1", "1,2,3,4", "Unsupported lookup: 'x'"),
    ],
)
def test_QueryStringFilter(fixtures, op, expected, error, caplog):
    f = QueryStringFilter(None, {"qs": op}, DemoModelField, None)
    result = f.queryset(None, DemoModelField.objects.all())
    value = list(result.values_list("unique", flat=True))
    assert value == expected.split(","), value
    assert f.error_message == error


def test_media():
    assert QueryStringFilter.factory(title="Title")(
        None, {}, DemoModelField, None
    ).media
